package SOF2Logger::Config;
use strict;

use File::Basename qw/basename dirname/;
use File::Copy;
use File::Find qw/find/;
use File::Glob ':glob';
use SOF2Logger::ServerStats;
use SOF2Logger::Utils qw/show_do dprint/;

### Config is a singleton pattern, only one can exist for a given stats pars
our $instance = 0;

# Now, instead of calling new, you will have to call "instance", 
# which gives you an instance of the config class.

# also, the first time you call instance, you can pass
# command_line_parameters to it, as you'll be setting up the object
# the first time.
sub instance
{
    my $self = shift;
    if($instance == 0) {
        $instance = SOF2Logger::Config->new(@_);
    }

    return $instance;
}

# ... method_maker would save all these lines...
sub minimum_time {my $self=shift; return $self->get_set('minimum_time', @_)}
sub minimum_kills {my $self=shift; return $self->get_set('minimum_kills', @_)}

sub top_weap {my ($self)=@_; return $self->{_config}->{top_weap}}
sub colorize_names {my ($self)=@_; return $self->{_config}->{name_colors} =~ /yes/i}
sub last_line_num {my $self=shift; return $self->{_config}->{log_file_last_line_num}}
sub help {my $self=shift; return $self->{_config}->{help}}
sub log_file {my $self=shift; return $self->{_config}->{log_file}}
sub debug {my $self=shift; return ($self->{_config}->{debug})}
sub reinit {my $self=shift; return $self->{_config}->{reinit}}
sub backup {my $self=shift; return $self->{_config}{backup_logfile} && $self->{_config}->{backup_logfile} =~ /yes/i}
sub top_num {my $self=shift; return $self->{_config}->{top_num}}
sub game_type {my $self=shift; return $self->get_set('game_type', @_)}
sub game_mod {my $self=shift; return $self->{_config}->{game_mod}}

=item full_filename

This method includes the directory of the script as well as the
filename itself.  That way, if the script is being called from a
different directory, it will continue to reference files as if it was
running from the scripts home directory

=cut

sub full_filename {
  my ($self, $file) = @_;

  return $file
    ? join '/', dirname($0), $file
      : '';
}

sub new {
  if (caller() ne 'SOF2Logger::Config') {
    die "Err: you cannot create an instance of this class\n";
  }

  my $class = shift;
  my $self = {};
  bless($self, $class);
  $self->init(@_);
  return $self;
}

sub init {
  my $self = shift;

  $self->{_config} = {};
  $self->{_config}->{image_dir} = 'ig';
  $self->{_config}->{css_dir} = 'css';
  $self->parse_command_line_switches(@_);
  $self->read_config_file();
  $self->gen_filename_getters();
  $self->read_filtered_players_into_config();
  $self->read_aliased_players_into_config();

  if ($self->reinit) {
    print "Initializing datafile\n" if $self->debug;
    $self->init_datafile();
  }
}


sub get_set
{
    my ($self, $method, $attrib) = @_;
    defined $attrib
        ? $self->{_config}{$method} = $attrib
        : return $self->{_config}{$method};
}


sub parse_command_line_switches
{
    my ($self, $command_line_switches) = @_;
	$self->{_config}->{config_file} = $$command_line_switches{config_file}
        ? $$command_line_switches{config_file}
        : $self->full_filename('config/logparse.conf');
	$self->{_config}->{log_file} = $$command_line_switches{log_file};
	$self->{_config}->{reinit} = $$command_line_switches{reinit};
	$self->{_config}->{debug} = $$command_line_switches{debug};
	$self->{_config}->{help} = $$command_line_switches{help};

    if(!$self->{_config}{config_file} && ! -f 'logparse.conf') {
        $self->{_config}->{help} = 1;
    }
}

sub _make_dir_if_not_exists {
  my ($self, $dir) = @_;
  if (!-d $dir) {
    print "creating output directory ($dir)\n";
    mkdir $dir,0755 or die "Err: couldn't create $dir :$!\n";
  }
}

sub read_aliased_players_into_config {
  my $self = shift;

  return unless -f $self->player_alias_filename;

  open (ALIAS, $self->player_alias_filename)
    or warn "Could not open player alias file "
      . $self->player_alias_filename . " for read: $!\n";

  while (<ALIAS>) {
    chomp;
    next if m/^#/;
    s/\s+//g;
    my ($from_id, $to_id) = split /:/, $_;
    $self->{_config}{aliased_players}{$from_id} = $to_id;
  }

  close ALIAS;
}

sub read_filtered_players_into_config
{
  my $self = shift;

  if (-f $self->filter) {
    # Read in the names in this list, push them into the filter
    open (FILTER, $self->filter)
      or die("Er: Could not open filter file @{[$self->filter]}: $!\n");

    while (<FILTER>) {
      chomp;
      # remove spaces
      s/\s*//;
      $self->{_config}{filtered_players}{uc($_)}++;
    }
    close(FILTER);
  }
}

sub check_and_copy_resources {
  my $self = shift;

  # check player template
  -f $self->player_template or
    die "Err: could not open file ". $self->player_template .": $!\n";

  -f $self->main_template or
    die 'Err: could not open file '. $self->main_template .": $!\n";

  -f $self->log_file or
    warn 'Err: could not open log file '. $self->log_file .": $!\n";

  $self->_make_dir_if_not_exists($self->output_dir);
  $self->_make_dir_if_not_exists($self->output_dir . '/' . $self->player_data_dir);
  $self->_make_dir_if_not_exists($self->output_dir .'/' . $self->css_dir);
  $self->_make_dir_if_not_exists($self->data_dir);
  $self->_make_dir_if_not_exists($self->output_dir . '/' . $self->image_dir);

  $self->copy_resources_to_output_dir();
}

sub copy_resources_to_output_dir {
  my ($self) = @_;

  my $output_dir = $self->output_dir;

  my $starting_dir = $self->full_filename('resources/default');

  warn "starting in $starting_dir\n";

  show_do('copying default resources',
	  sub { find({wanted => sub {
                    my $dir = $File::Find::dir;
                    $dir =~ s|.*resources/default||;
                    $dir = $output_dir . $dir;
                    next unless -f $_;
                    next if $File::Find::name =~ /.svn/;
                    warn "$File::Find::name $dir " . basename $_ . "\n" if $self->debug;
                    copy($_, "$dir/" . basename $_)
                      or warn "Could not copy $_ to $dir/$_ : $!\n"},
                  no_chdir=>1}
                 ,
                  $self->full_filename('resources/default')) });

  show_do('copying game-specific resources',
	  sub { find({wanted => sub {
                    my $dir = $File::Find::dir;
		    my $game_type = $self->game_type;
                    $dir =~ s|.*resources/$game_type||e;
                    $dir = $output_dir . $dir;
                    next unless -f $_;
                    next if $File::Find::name =~ /CVS/;
                    warn "$File::Find::name $dir " . basename $_ . "\n" if $self->debug;
                    copy($_, "$dir/" . basename $_)
                      or warn "Could not copy $_ to $dir/$_ : $!\n"},
                  no_chdir=>1}
                 ,
                  $self->full_filename('resources/' . $self->game_type)) });
}

#############################################################
# -- READ IN THE CONFIGURATION FILE -- 						#
# returns a hash reference containing all the config values.#
#############################################################
sub read_config_file
{
    my ($self) = @_;
    my %conf_options = (
        server_admin => '^\s*Administrator\s+(.+)',
        server_admin_email   => '^\s*AdminEmail\s+(\S+)',
        server_name => '^\s*ServerName\s+(.+)',
        top_num => '^\s*TOPNUM\s+(\d+)',
        top_weap => '^\s*TOPWEAP\s+(\d+)',
        minimum_kills => '^\s*MINIMUM_KILLS\s+(\d+)',
        minimum_time => '^\s*MINIMUM_TIME\s+(\d+)',
        output_file => '^\s*OUTPUT_FILENAME\s+(\S+)',
        kph_file => '^\s*KPH_FILENAME\s+(\S+)',
        deaths_file =>'^\s*DEATHS_FILENAME\s+(\S+)',
        eff_file => '^\s*EFF_FILENAME\s+(\S+)',
        suicides_file => '^\s*SUICIDES_FILENAME\s+(\S+)',
        captures_file => '^\s*CAPTURES_FILENAME\s+(\S+)',
        returns_file => '^\s*RETURNS_FILENAME\s+(\S+)',
        log_file => '^\s*LOGFILE\s+(\S+)',
        timeplayed_file =>'^\s*TIMEPLAYED_FILENAME\s+(\S+)',
        main_template =>  '^\s*MAIN_TEMPLATE\s+(\S+)',
        player_template =>'^\s*PLAYER_TEMPLATE\s+(\S+)',
        output_dir => '^\s*OUTPUTDIR\s+(\S+)',
        data_dir =>'^\s*DATADIR\s+(\S+)',
        player_data_dir => '^\s*PLAYERDATADIR\s+(\S+)',
        stylesheet => '^\s*STYLESHEET\s+(\S+)',
        filter => '^\s*FILTER\s+(\S+)',
        name_colors => '^\s*NAME_COLORS\s+(\S+)',
        log_file_last_line_num => '^\s*LOG_LAST_LINE_NUM\s+(\d+)',
        backup_logfile => '^\s*BACKUP\s+(\S+)',
        game_type => '^\s*GAMETYPE\s+(\S+)',
        game_mod => '^\s*GAMEMOD\s+(\S+)',
        player_alias => '^\s*PLAYER_ALIAS\s+(\S+)',
    );

    my $fh;
    warn "using config file " . $self->{_config}{config_file};
    open ($fh, $self->{_config}{config_file}) or
      do { warn "Err: could not open ".$self->{_config}{config_file} ." for read: $!";
           return; };

    my %server_stats_options = ( server_admin => 1, server_admin_email => 1, server_name => 1 );

    while (my $line = <$fh>) {
        next if $line =~ /^#/;
        next if $line =~ /^\s*$/;

        while(my($key, $value) = each %conf_options) {
            next if($key eq 'log_file' && ($self->log_file));
            if($line =~ $value){
                my $data = $1;
                chomp $data;

                if ($server_stats_options{$key}) {
                    SOF2Logger::ServerStats->$key($data);
                } else {
                    $self->{_config}->{$key} = $data;
                }
            }
        }
    }
    close $fh;
}

# this method generates the getter methods that we use to access the
# various filenames we keep track of via the Config file
sub gen_filename_getters {
  my $self = shift;
  my $config = $self->{_config};

  # if a key exists, but has no value, we assume it directly maps to a config key.
  my %short_filename = ( output_file   => '',
                         main_template => sub { $self->full_filename($config->{main_template})},
                         player_template => sub {$self->full_filename($config->{player_template})},
                         data_dir => sub {$self->full_filename($config->{data_dir})},
                         # you can reference previously defined
                         # methods, as the methods are not evaluated
                         # until they're needed
                         player_file => sub { $self->data_dir . '/players.dat' },
                         kills_file => sub { $self->data_dir . '/kills.dat' },
                         server_stats_file => sub { $self->data_dir . '/server_stats.dat' },
                         filter => sub { $self->full_filename($config->{filter})},
                         timeplayed_file => '',
                         eff_file => '',
                         kph_file => '',
                         image_dir => '',
                         css_dir => '',
                         config_file => '',
                         output_dir => '',
                         player_data_dir => '',
                         kill_file => '',
                         suicides_file => '',
                         captures_file => '',
                         returns_file => '',
                         deaths_file => '',
                         stylesheet => '',
                         player_alias_filename => sub { $self->full_filename($config->{player_alias})},
                       );
  no strict 'refs';

  foreach my $method (keys %short_filename) {
    *{q/SOF2Logger::Config::/ . $method} = sub {
      my ($self, $value) = @_;

      if (defined $value) {
        $self->{_config}{$method} = $value;
      }

      # intercept calls to the config object and optionally execute
      # our code there
      return ref $short_filename{$method}
        ? $short_filename{$method}->()
          : $self->{_config}{$method};
    }
  }
}

# if we have an alias, it returns it, otherwise it returns the id passed in
sub player_alias {
  my ($self, $id) = @_;
  return defined $self->{_config}{aliased_players}{$id}
    ? $self->{_config}{aliased_players}{$id}
      : $id;
}

sub is_player_filtered
{
    my ($self, $player) = @_;

    foreach my $filter (@{$self->filtered_players}) {
        return 1 if uc($player->id) eq uc($filter)
        || uc($player->name) eq uc($filter);
    }
}

sub filtered_players
{
    my ($self) = @_;
    my @players = keys %{$self->{_config}{filtered_players}};
    return \@players;
}


sub init_datafile
{
    my $self = shift;
    unlink($self->player_file, $self->kills_file, $self->server_stats_file);
}

sub delete_old_player_files
{
    my ($self) = @_;
    my @filelist = bsd_glob(
        $self->output_dir ."/". $self->player_data_dir(). "/*.html");
    unlink @filelist;
}

1;
