package SOF2Logger::Kills;

use strict;
use SOF2Logger::Utils qw/min_defined/;
use SOF2Logger::Config;
use SOF2Logger::Player;
use SOF2Logger::ELORating qw/update_rating/;

=head1 NAME

Kills.pm : the kills object

=head1 DESCRIPTION

Stores all the kills for all the players.

=head2 PUBLIC INTERFACE

=over 8

=cut

# TODO: Suicides could be represented as kills where kill_id = vic_id,
# this would let us track what weapon the person suicided with, and we
# could do "funny" things like "Most dangerous weapon to user"

# We would also have to exclude suicides from the list of player
# deaths, as it would artificially raise their deaths.

my $BY_KILLER_BY_WEAPON = {};
my $BY_KILLER_BY_VICTIM = {};
my $BY_KILLER_BY_LOCATION = {};
my $BY_WEAPON_BY_KILLER = {};
my $BY_VICTIM_BY_KILLER = {};
my $BY_KILLER = {};
my $BY_WEAPON = {};
my $DEATHS_BY_PLAYER = {};
my $SUICIDES_BY_PLAYER = {};
my $IS_FILTERED_PLAYER = {};
my $total_deaths = 0;
my $total_kills = 0;

sub add_kill
{
    # when we add a kill we have to add it to all the index tables.  This makes
    # it faster.
    my ($class, $killer, $victim, $weapon, $location, $time) = @_;

    update_rating(SOF2Logger::Player->find($killer), SOF2Logger::Player->find($victim));

    $BY_WEAPON->{$weapon}++;
    $BY_KILLER->{$killer}++;
    $BY_KILLER_BY_WEAPON->{$killer}{$weapon}++;
    $BY_KILLER_BY_VICTIM->{$killer}{$victim}++;
    $BY_KILLER_BY_LOCATION->{$killer}{$location}++;
    $BY_WEAPON_BY_KILLER->{$weapon}{$killer}++;
    $BY_VICTIM_BY_KILLER->{$victim}{$killer}++;
    $DEATHS_BY_PLAYER->{$victim}++;
    $total_kills++;
    $total_deaths++;
}

sub is_filtered_player
{
    my ($class, $player, $val) = @_;

    defined $val
        ? $IS_FILTERED_PLAYER->{$player} = $val
        : return defined $IS_FILTERED_PLAYER->{$player};
}

sub add_suicide
{
    my ($class, $victim) = @_;
    $SUICIDES_BY_PLAYER->{$victim}++;
}

sub kills_for_player
{
    my ($class, $player) = @_;
    return $BY_KILLER->{$player} || 0;
}

sub headshots_for_player
{
    my ($class, $player) = @_;
    my $headshot_loc = 'head';
    return $BY_KILLER_BY_LOCATION->{$player}{$headshot_loc} || 0;
}

sub kills_by_weapon
{
    my ($class, $limit_to_kills_by_other_players) = @_;
    my $by_weapon_ref = $class->_sorted_ref($BY_WEAPON, 0);
    my @good_weapons;

    if ($limit_to_kills_by_other_players) {
        foreach my $weapon_info (@$by_weapon_ref) {
            my $ref = $class->kills_by_player_for_weapon($weapon_info->[0], 1);
            next unless scalar(@$ref) > 0;
            push @good_weapons, $weapon_info;
        }
    } else {
        @good_weapons = @$by_weapon_ref;
    }
    return \@good_weapons;
}

sub kills_by_player_for_player
{
    my ($class, $name) = @_;
    return $class->_sorted_ref($BY_KILLER_BY_VICTIM->{$name}, 0);
}

sub kills_by_player_for_weapon
{
    my ($class, $weapon, $filter_players) = @_;
    return $class->_sorted_ref($BY_WEAPON_BY_KILLER->{$weapon}, $filter_players);
}

sub kills_by_weapon_for_player
{
    my ($class, $player) = @_;
    return $class->_sorted_ref($BY_KILLER_BY_WEAPON->{$player}, 0);
}

sub kills_for_weapon
{
    my ($class, $weapon) = @_;
    return scalar $BY_WEAPON->{$weapon} || 0;
}

sub deaths_for_player
{
    my ($class, $player) = @_;
    return $DEATHS_BY_PLAYER->{$player} || 0;
}

sub weapons
{
    my ($class) = @_;
    my @weapons = sort { uc $a cmp uc $b } keys %$BY_WEAPON;
    return \@weapons;
}

sub deaths_by_player_for_player
{
    my ($class, $player) = @_;
    return $class->_sorted_ref($BY_VICTIM_BY_KILLER->{$player}, 0);
}

sub suicides_for_player
{
    my ($class, $player) = @_;
    return $SUICIDES_BY_PLAYER->{$player} || 0;
}

# helper method sorts a hash ref by its numeric values and returns an array
# ref of array refs containing the hash key and its value
# ex. (the return from _sorted_ref)
# [
#   [ 'bobby', 2 ],
#   [ 'billy', 1 ],
#   [ 'fred', 0  ],
# ]
sub _sorted_ref
{
    my ($class, $ref, $should_filter_results) = @_;
    my @filtered_info;

    foreach my $info ($class->_info_by_value($ref)) {
        push @filtered_info, $info 
            unless $IS_FILTERED_PLAYER->{$$info[0]}
                   && $should_filter_results;
    }

	return \@filtered_info;
}

sub _info_by_value
{
    my ($class, $ref) = @_;

    return  map { [$_, $$ref{$_}] }
        sort { $$ref{$b} <=> $$ref{$a} } keys %$ref;
}

sub total_deaths { return $total_deaths }
sub total_kills { return $total_kills }

sub delete_all {
  $BY_KILLER_BY_WEAPON = {};
  $BY_KILLER_BY_VICTIM = {};
  $BY_KILLER_BY_LOCATION = {};
  $BY_WEAPON_BY_KILLER = {};
  $BY_VICTIM_BY_KILLER = {};
  $BY_KILLER = {};
  $BY_WEAPON = {};
  $DEATHS_BY_PLAYER = {};
  $SUICIDES_BY_PLAYER = {};
  $IS_FILTERED_PLAYER = {};
  $total_deaths = 0;
  $total_kills = 0;
}

sub thaw {
  my ($class, $file) = @_;
  return unless -f $file;

  open (PLAYERFILE, $file)
    or die "Err: could not open file: $file : $!";

  while (my $line = <PLAYERFILE>) {
    chomp $line;

    next if $line =~ m/^$/;

    my ($code, $data) = split /:/, $line, 2;

    # remove any trailing color codes that do nothing but screw up stats
    # these typically occur with OSP, I don't know why
    $data =~ s/(\^7)+\t/\t/;

    # This occurs once for each player, so we will use it to generate our
    # players hash from.
    # PLAYER_NAME\tKILLS\tDEATHS
    if ($code eq 'P') {
      my ($id, $killed, $deaths) = split /\t/, $data;
      $BY_KILLER->{$id} = $killed;
      $DEATHS_BY_PLAYER->{$id} = $deaths;
      $total_kills += $killed;

      # W:WEAPON_NAME\tTIMES_USED
    } elsif ($code eq 'W') {
      my ($weapon, $times_used) = split /\t/, $data;
      $times_used =~ tr/ //d;
      $BY_WEAPON->{$weapon} = $times_used;

      # TP:PLAYER_NAME\tWEAPON\tTIMES_USED\tHEADSHOTS
    } elsif ($code eq 'TP') {
      my ($id, $weapon, $times_used, $headshots) = split /\t/, $data;
      $times_used =~ tr/ //d;
      $headshots =~ tr/ //d;
      $BY_KILLER_BY_WEAPON->{$id}{$weapon} = $times_used;
      $BY_KILLER_BY_LOCATION->{$id}{head} = $headshots;
      $BY_WEAPON_BY_KILLER->{$weapon}{$id} = $times_used;

      # PV:KILLER_NAME\tVICTIM_NAME\tTIMES_KILLED
    } elsif ($code eq 'PV') {
      my ($killer_id, $victim_id, $times_killed) = split /\t/, $data;
      $times_killed =~ tr/ //d;
      $BY_KILLER_BY_VICTIM->{$killer_id}{$victim_id} = $times_killed;
      $BY_VICTIM_BY_KILLER->{$victim_id}{$killer_id} = $times_killed;

      # S:PLAYER_NAME\tSUICIDES
    } elsif ($code eq 'S') {
      my ($id, $suicides) = split /\t/, $data;
      $suicides =~ tr/ //d;
      $SUICIDES_BY_PLAYER->{$id}= $suicides;
    } else {
      print "datafile at $. contains strange line $line \n";
    }
  }
  close(PLAYERFILE);
}

sub freeze {
  my ($class, $file) = @_;

  open (PLAYEROUT, '>', $file) or
    die "Err: Could not open datafile ($file) for write:$!";

  # Calculate and write out the player data file
  # P:PLAYER_NAME\tKILLS\tDEATHS
  foreach my $player (SOF2Logger::Player->find_all()) {
		my $kills = $class->kills_for_player($player->id);
		my $deaths = $class->deaths_for_player($player->id);
    print PLAYEROUT "P:@{[$player->id]}\t$kills\t$deaths\n";
  }

  # W:WEAPON_NAME\tTIMES_USED
  print PLAYEROUT map {"W:$$_[0]\t$$_[1]\n"} @{$class->kills_by_weapon};

  # TODO: resolve headshots, since I no longer track them by weapon
  # TP:PLAYER_NAME\tWEAPON\tTIMES_USED\tHEADSHOTS
  foreach my $player (SOF2Logger::Player->find_all()) {
    my $kills_info = $class->kills_by_weapon_for_player($player->id);
    # its a little waste of space, but fixes the saving of headshot stats, so save headshots
    # for every weapon, writing the same value every time
    my $headshots = $class->headshots_for_player($player->id);
    print PLAYEROUT map {"TP:@{[$player->id]}\t$$_[0]\t$$_[1]\t$headshots\n"} @$kills_info;
  }

  # PV:KILLER_NAME\tVICTIM_NAME\tTIMES_KILLED
  foreach my $player (SOF2Logger::Player->find_all()) {
    my $kills_info = $class->kills_by_player_for_player($player->id);

    print PLAYEROUT map {
      SOF2Logger::Player->find($$_[0])
	  ? "PV:@{[$player->id]}\t@{[SOF2Logger::Player->find($$_[0])->id]}\t$$_[1]\n"
	    : "Err: could not find victim: $$_[0]" } @$kills_info;
  }

  # S:PLAYER_NAME\tSUICIDES
  print PLAYEROUT
		map {"S:" . join("\t", $_, $class->suicides_for_player($_)) . "\n" }
			map { $_->id } SOF2Logger::Player->find_all();

  close PLAYEROUT;
}

1;
