package SOF2Logger::Parser;

use strict;
use SOF2Logger::Utils qw/create_get_set/;
use SOF2Logger::Kills;
use SOF2Logger::Player;
use SOF2Logger::ServerStats;

=head

2 major parts of the logs
    1: The server stats
        * The name of the server
        * The server admin
        * Mods that have been played
        * maps that have been played
        * total time of the server
        * total number of kills, deaths, suicides, etc
        * lines processed

    2: The player stats
        * kills for player
        * deaths for player
        * weapons used for player
        * weapons used on player
        * suicides
        * total time

=cut

sub new
{
    my ($class, $filename, $debug) = @_;
    my $self = {};
    bless $self, $class;

    my @methods = qw/debug log last_known_good_time/;

    foreach my $method (@methods) {
      create_get_set($class, $method);
    }

    $self->log($filename);
    $self->debug($debug);
    $self->init();

    return $self;
}

sub init {
  my ($self) = @_;

  $self->{weapon_map} = $self->get_weapon_map();
  $self->{color_map} = $self->get_color_map();
  $self->{events} = $self->get_events();
}

sub events {
  my ($self, $event) = @_;
  return $self->{events}{$event} if ($event);
}

# virtual methods
sub get_events {}
sub record_kill { die "implement " . (caller(0))[3] . "\n"; }
sub change_info_client { die "implement ".(caller(0))[3]. "\n"; }
sub shutdown_game { die "implement ".(caller(0))[3]. "\n"; }
sub init_game { die "implement ".(caller(0))[3]. "\n"; }
sub disconnect_client { die "implement ".(caller(0))[3]. "\n"; }
sub connect_client { die "implement ".(caller(0))[3]. "\n"; }
sub parse_line { return 0; }
sub get_weapon_map { return {} }
sub get_color_map { return {} }

# should probably be over-written in child class
sub unhandled_line { my ($self, $line) = @_; die "unhandled line: '$line'\n" }

# if the modtimes match, then we skip the file, otherwise, we read the file in
# once to see if we can get back to our last line.
sub get_checked_filehandle
{
    my ($self) = @_;
    my @stat = stat($self->log);
    my $new_modtime = $stat[9];
    my $stored_modtime = SOF2Logger::ServerStats->log_modtime;
    my $fh;

    # we don't need to read from the file
    if ($stored_modtime && $stored_modtime == $new_modtime) {
      warn "modtimes same, not parsing logfile\n" if $self->debug;
        $fh = "0";
    } else {
        # try looking through the logfile to see if we have the same line
        # on a file that has a different modtime
        $fh = $self->open_log;
        my $last_line = SOF2Logger::ServerStats->last_line;
        my $last_line_num = SOF2Logger::ServerStats->last_line_num;
        my $current_line_num = 0;
        my $line;
        while ($current_line_num < $last_line_num && ($line = <$fh>)) {
            $current_line_num++;
        }
        $line ||= '';

        chomp $line;

        # if they don't match, return a new filehandle
        if ($line ne $last_line || $current_line_num != $last_line_num) {
            warn "Starting from the beginning of the log.\n" if $self->debug;
            close $fh;
            $fh = $self->open_log;
        }

    }
    return $fh;
}

#accessors/mutators
sub parse_log
{
    my ($self) = @_;
    my $num_lines = 0;
    my $line;
    my $fh = $self->get_checked_filehandle;
    my @stat = stat($self->log);
    my $modtime = $stat[9];
    my $last_parsed_line;

    # having no filehandle means that the last file parsed and this file
    # are the same, so we don't need to parse it again
    return -1 unless $fh;

    while ($line = <$fh>) {
      my ($time, $event, $info) = $self->parse_line($line);
      my $meth_name = ($event && $self->events($event))
	? $self->events($event)
	  : 'unhandled_line';

      $self->last_known_good_time($time);
      $self->$meth_name($time, $info);

      $num_lines++;
      $last_parsed_line = $line;
    }

    $self->shutdown_game($self->last_known_good_time);

    close($fh);

    SOF2Logger::ServerStats->last_line($last_parsed_line);
    SOF2Logger::ServerStats->log_modtime($modtime);
    SOF2Logger::ServerStats->last_line_num($num_lines);
    SOF2Logger::ServerStats->add_total_lines($num_lines);
    $self->set_total_time;
    # stores the last line we have seen, but keeps track of the total number
    # of lines that have been processed for as far as the tracking goes back
}

sub set_total_time
{
    my ($self) = @_;
    my $total_time = 0;
    foreach my $player (SOF2Logger::Player->find_all()) {
        $total_time += $player->total_time;
    }
    SOF2Logger::ServerStats->total_time($total_time);
}

sub open_log
{
    my ($self) = @_;
    my $fh;

    open( $fh, '<', $self->log) 
        or die "Err: could not open log: ".$self->log.": $!\n";

    return $fh;
}

1;
