package SOF2Logger::Parser::COD;

use strict;
use warnings;

use base 'SOF2Logger::Parser::SOF2';

use SOF2Logger::Kills;
use SOF2Logger::Player;
use SOF2Logger::Parser::SlotList;

use constant SlotList => 'SOF2Logger::Parser::SlotList';
use constant Utils => 'SOF2Logger::Utils';

# NOTES: it looks like we can determine the version based on the
# shortversion key in the init line.

# This module works for the next gen of COD logs (1.3/1.4/1.5/+).
# This module also works for COD 2!

# Unforutunately, I see no way to determine that we are running the
# ROC mod.

# TODO: It appears COD doesn't record name change lines, so you should
# always attempt to use the slot, and then fall back to the name.

sub get_events {
  return { InitGame => 'init_game',
           J => 'connect_client',
           K => 'record_kill',
           Q => 'disconnect_client',
           ExitLevel => 'shutdown_game',
           RestartGame => 'shutdown_game',
         };
}

sub parse_line
{
    # 35:01 J;111222;0;{LLC}Bunga^7
    my ($self, $line) = @_;

    my ($time, $command, $info);

    if ($line =~ / *(\d+):(\d)(\d?)\s?([^;:]+)(;|:)\s*(.*)/) {
        $time = ($1 * 60) + ($2 * 10) + ($3?$3:0);
        $command = $4;
        $info = $6;
    }

    return $time, $command, $info;
}

sub info_from_kill_line {
  my ($self, $info) = @_;

  my ($vic_uid,
      $vic_slot,
      $vic_team,
      $vic_name,
      $aggr_uid,
      $aggr_slot,
      $aggr_team,
      $aggr_name,
      $weap_name,
      $damage,
      $dam_source,
      $loc) = split /;/, $info;

  # for some reason world kills show the team as "world" with no name
  return ( $self->get_id($aggr_name || $aggr_team,
			 SlotList->get_slot($aggr_slot)),
	   $self->get_id($vic_name, SlotList->get_slot($vic_slot)),
	   $self->weapon_map($weap_name) );

}

sub is_headshot_kill {
  my ($self, $info) = @_;
  my @info = split /;/, $info;
  return $info[-1] eq 'head';
}

sub init_game_hook {
  my ($self, $time, $info) = @_;
  my %init_info = ( split /\\/, substr($info, 1) );

  SOF2Logger::Config->instance()->admin($init_info{_Admin});
  SOF2Logger::Config->instance()->admin_email($init_info{_Email});
}

sub change_infoo_client { # TODO: These appear not to get logged in COD
}

sub disconnect_client {
    my ($self, $time, $info) = @_;
    my ($uid, $slot, $name) = split /;/, $info;
    SlotList->disconnect($slot, $time)
}

sub connect_client
{
    my ($self, $time, $info) = @_;

    my @info = split(/;/, $info, 3);

    # for certain versions of cod, we have a UID, for other versions we don't.
    # Therefore, if we have 3 elements, the first is the uid, if we only have 2,
    # the first element is the name
    my ($id, $slot, $name);
    if (@info == 2) {
        ($slot, $name) = @info;
        $id = uc(Utils->clean_q3_color_string($name));
    } else {
      ($id, $slot, $name) = @info;
      # sometimes, uids are disabled, so we have to use the name instead
      if ($id == 0) {
        $id = uc(Utils->clean_q3_color_string($name));
      }
    }

    my $player = SOF2Logger::Player->find_or_create( name => $name, id => $id );

    SOF2Logger::Parser::SlotList->put_player_in_slot($slot,
						     $player,
						     0,
						     $time,
						     0);
}

sub get_weapon_map {
  return
    {
     mp44_mp => 'MP44',
     MOD_RIFLE_BULLET => 'MP44',
     shotgun_mp => 'Shotgun',
     thompson_mp  => 'Thompson Submachine Gun',
     ppsh_mp => 'PP SH',
     m1garand_mp => 'M1 Garand',
     luger_mp => 'Luger',
     frag_grenade_german_mp => 'Stick bomb',
     colt_mp => 'Colt',
     frag_grenade_american_mp => 'Frag Grenade (USA)',
     kar98k_mp => 'KAR 98K',
     mg42_bipod_stand_mp => 'MG42 (bipod stand)',
     TT30_mp => 'TT30',
     webley_mp => 'Webley',
     greasegun_mp => 'Grease Gun',
     bren_mp => 'Bren',
     mosin_nagant_mp => 'Mosin Nagant',
     frag_grenade_british_mp => 'Frag Grenade (British)',
     g43_mp => 'G43',
     frag_grenade_russian_mp => 'Frag Grenade (Russian)',
     bar_mp => 'Bar',
     mp40_mp => 'MP40',
     '30cal_stand_mp' => '30 cal (stand)',
     PPS42_mp => 'PPS42',
     sten_mp => 'Sten',
     enfield_mp => 'Enfield',
     m1carbine_mp => 'M1 Carbine',
    };
}

1;

