package SOF2Logger::Parser::JK2;

use strict;

use base q/SOF2Logger::Parser::Q3A/;

use Carp qw/confess/;
use SOF2Logger::Kills;
use SOF2Logger::Parser::SlotList;
use SOF2Logger::Player;
use SOF2Logger::Utils qw/create_get_set/;

use constant SlotList => 'SOF2Logger::Parser::SlotList';
use constant Utils => 'SOF2Logger::Utils';

sub WORLD_ID { 'world' };

sub new {
  my $class = shift;
  my $self = $class->SUPER::new(@_);

  $self->last_known_good_time(-1);
  $self->init_flags;
  return $self;
}

sub init_flags {
    my ($self) = @_;

    $self->{flags} = {};
    my $flags = $self->{flags};
    my $blue = $self->BLUE_TEAM;
    my $red = $self->RED_TEAM;

    $flags->{$blue} = [ 'SPAWN', 0 ];
    $flags->{$red}  = [ 'SPAWN', 0 ];
}

sub init_game_hook {}

sub get_events {
  return { InitGame => 'init_game',
           Kill => 'record_kill',
           ShutdownGame => 'shutdown_game',
           ClientUserinfoChanged => 'change_info_client',
           ClientDisconnect => 'disconnect_client',
           ClientConnect => 'connect_client',
           Item => 'record_item',
           setteam => 'change_team_client',
         };
}

sub change_team_client {
  my ($self, $time_in_seconds, $info) = @_;

  # this is here because setteam lines use the string version of the
  # team name, while the ClientUserinfoChanged lines use the numerical
  # version of the team name. Why this is, I don't know.
  my $team_str_to_num_xref = { BLUE => $self->BLUE_TEAM,
			       RED  => $self->RED_TEAM,
			       FREE => $self->SPECTATOR_TEAM };

  my ($num, $from_team, $to_team) = split / /, $info;
  my $is_spectating = 1
    if $team_str_to_num_xref->{$to_team} == $self->SPECTATOR_TEAM;

  SlotList->put_player_in_slot($num,
			       SlotList->get_player($num),
			       $team_str_to_num_xref->{$to_team},
			       $time_in_seconds,
			       $is_spectating)
    if SlotList->get_player($num);
}

sub info_from_kill_line {
  my ($self, $data) = @_;

  # $line =~ /Kill: (\d+) (\d+) (\d+): (.*?) killed (.*?) by (MOD_\w+)/o;
  # do some fancy splitting to cut down on the use of regular expressions
  my ($short_info, $long_info) = split /:/, $data, 2;
  my ($killer_slot, $victim_slot, $weapon_num) = split ' ', $short_info, 3;

  $long_info =~ / (.*?) killed (.*?) by (MOD_\w+)/;
  my $killer_name = $1;
  my $victim_name = $2;
  my $weapon_id = $3;

  my $vic_slot = SlotList->get_slot($victim_slot);
  $self->drop_flag($vic_slot) if $vic_slot;
  # TODO: give the killer points for killing the flag carrier

  return ( $self->get_id($killer_name, SlotList->get_slot($killer_slot)),
	   $self->get_id($victim_name, SlotList->get_slot($victim_slot)),
	   $self->weapon_map($weapon_id) );
}

# sample suicides
# 309:39 Kill: 7 7 35: ^1^2Gothic killed ^1^2Gothic by MOD_SUICIDE
# 666:57 Kill: 1022 8 34: <world> killed ^7kibbel by MOD_FALLING
sub is_suicide {
  my ($self, $killer_id, $victim_id, $weapon_name) = @_;
  return ( $killer_id eq $self->WORLD_ID || $killer_id eq $victim_id )

}

sub is_headshot_kill { 0 }

# Sample line
#160:49 ClientUserinfoChanged: 4 n\^1^1^7.^7^7^7cetnyx\t\1\model\jeditrainer/default\g_redteam\Empire\g_blueteam\Rebellion\c1\3\c2\5\hc\100\w\0\l\0\tt\0\tl\0
sub change_info_client {
  my ($self, $time_in_seconds, $info) = @_;

  my ( $slot_num, $player_name, $team, $avatar_name )
    = $info =~ m|(\d+) n\\(.+?)\\t\\(\d)\\model\\([^/]*)|;

  # there appears to be a bug in jk2 that doesn't write
  # ClientUserinfoChanged lines properly, so we can skip lines that
  # don't have all the info we need
  unless (defined($slot_num) && defined($player_name) && defined($team)) {
    return;
  }

  my $id = $self->get_id($player_name, SlotList->get_slot($slot_num));

  my $player =
    SOF2Logger::Player->find_or_create(id => $id, name => $player_name);

  SlotList->put_player_in_slot($slot_num,
			       $player,
			       $team,
			       $time_in_seconds,
			       ($team == $self->SPECTATOR_TEAM() ? 1 : 0));

  # TODO: Track time for each alias
  $player->avatar_name($avatar_name . ".jpg") if $avatar_name;
}

sub reset_game_state {
  my ($self, $time) = @_;

  foreach my $slot (SlotList->all_slots()) {
    SlotList->disconnect($slot->num, $time);
  }

  $self->init_flags;
}

sub disconnect_client {
    my ($self, $time, $info) = @_;
    (my $num) = $info =~ /(\d+)/;
    SlotList->disconnect($num, $time);

    # if, for some reason, he was carrying the flag, drop it
    $self->drop_flag(SlotList->get_slot($num));
}

# do nothing for connects, since they are all handled by ClientUserInfoChanged
sub connect_client { return 0; }

# this function has to handle flag caps and returns
# needs to keep map state
# if the flag color matches the player->slot->team, its a return
# 176:50 Item: 8 team_CTF_redflag
# a state machine monitors where the flag is in the map, and what the 
#  next possible state is.
sub record_item {
    my ($self, $time_in_seconds, $data) = @_;
    my ($slot_num, $flag) = split ' ', $data, 2;

    # we only handle team_CTF_* items right now
    $self->handle_flag(SlotList->get_slot($slot_num), $flag)
      if $flag =~ /team_CTF_/;
}

# In JK2 land, we don't have a UID, so we always use the stripped
# color name.  If we can't find the player based on name, only then
# will we use the slot.
sub get_id {
  my ($self, $name, $slot) = @_;
  my $id_from_name = uc(Utils->clean_q3_color_string($name));
  my $player = SOF2Logger::Player->find($id_from_name);

  if ($player) {
    return $player->id();
  } elsif($slot && $slot->player) {
    return $slot->player->id;
  }

  # if we haven't gotten the id yet, go ahead and create the player
  SOF2Logger::Player->find_or_create(id => $id_from_name);
  return $id_from_name;
}

sub drop_flag {
  my ($self, $slot) = @_;
  my $flags = $self->{flags};

  return unless $slot;
  return unless ($flags->{$slot->team}->[1] == $slot);

  # dropping enemy team flag
  my $enemy_team = ($slot->team == $self->RED_TEAM)
    ? $self->BLUE_TEAM
      : $self->RED_TEAM;
  $flags->{$enemy_team} = [ 'DROPPED', 0 ];
}

sub handle_flag {
  my ($self, $slot, $flag) = @_;

  return unless $slot;

  my $RED_FLAG = 'team_CTF_redflag';
  my $BLUE_FLAG = 'team_CTF_blueflag';

  # state machine for flag goes like this
  # flag @ home
  # enemy picks enemyflag with Item: X enemyflag
  # enemy flag now carried
  # next flag state either dropped or capped
  # if dropped, its by a Kill: X me
  #   flag might be returned by Item: X myflag
  # if capped MyFlag must be home, and cap shows by Item: X myflag
  #   flag returns to enemy home

  my $flags = $self->{flags};
  my $team = $slot->team();
  my $flag_team = (($flag =~ /redflag/) ? $self->RED_TEAM : $self->BLUE_TEAM);
  my $enemy_team = ($slot->team == $self->RED_TEAM)
    ? $self->BLUE_TEAM
      : $self->RED_TEAM;

  if ($slot->team() == $flag_team) {
    # if my flag is home, its a cap
    if ($flags->{$team}[0] eq 'SPAWN') {

      # double check I am carrying the enemy flag
      my $enemy_flag_slot = $flags->{$enemy_team}[1];
      unless ($enemy_flag_slot && $slot->num == $enemy_flag_slot->num) {
	warn "Err: player ". $slot->num
	  . " team: $team was not listed as carrying flag, but he captured it!";
      }

      $slot->player()->add_cap;
      # return the enemies flag
      $flags->{$enemy_team} = ['SPAWN', 0];
    } else {
      $slot->player()->add_return;
      $flags->{$slot->team} = ['SPAWN', 0];
    }
  } else {
    # carrying the enemy flag
    $flags->{$enemy_team} = ['CARRIED', $slot];
  }
}

sub get_weapon_map {
  return
    {
     'MOD_UNKNOWN'               =>  'Unknown',
     'MOD_STUN_BATON'            =>  'Stun Baton',
     'MOD_MELEE'                 =>  'Kick',
     'MOD_SABER'                 =>  'Saber',
     'MOD_BRYAR_PISTOL'          =>  'Bryar Pistol',
     'MOD_BRYAR_PISTOL_ALT'      =>  'Bryar Pistol Alt',
     'MOD_BLASTER'               =>  'Blaster',
     'MOD_DISRUPTOR'             =>  'Disruptor',
     'MOD_DISRUPTOR_SPLASH'      =>  'Disruptor Splash',
     'MOD_DISRUPTOR_SNIPER'      =>  'Disruptor Snipe',
     'MOD_BOWCASTER'             =>  'Bowcaster',
     'MOD_REPEATER'              =>  'Repeater',
     'MOD_REPEATER_ALT'          =>  'Repeater Plasma',
     'MOD_REPEATER_ALT_SPLASH'   =>  'Repeater Plasma Splash',
     'MOD_DEMP2'                 =>  'Mystery Weapon',
     'MOD_DEMP2_ALT'             =>  'Mystery Weapon Splash',
     'MOD_FLECHETTE'             =>  'Flechette',
     'MOD_FLECHETTE_ALT_SPLASH'  =>  'Flechette Splash',
     'MOD_ROCKET'                =>  'Rocket Launcher',
     'MOD_ROCKET_SPLASH'         =>  'Rocket Launcher Splash',
     'MOD_ROCKET_HOMING'         =>  'Rocket Homing',
     'MOD_ROCKET_HOMING_SPLASH'  =>  'Rocket Homing Splash',
     'MOD_THERMAL'               =>  'Thermal Detonator',
     'MOD_THERMAL_SPLASH'        =>  'Thermal Detonator Splash',
     'MOD_TRIP_MINE_SPLASH'      =>  'Tripmine',
     'MOD_TIMED_MINE_SPLASH'     =>  'Timed Mine',
     'MOD_DET_PACK_SPLASH'       =>  'Detonator Pack',
     'MOD_FORCE_DARK'            =>  'Dark Force',
     'MOD_SENTRY'                =>  'Sentry',
     'MOD_WATER'                 =>  'Drowned',
     'MOD_SLIME'                 =>  'Slimed',
     'MOD_LAVA'                  =>  'HotLavad',
     'MOD_CRUSH'                 =>  'Crushed',
     'MOD_TELEFRAG'              =>  'Telefragged',
     'MOD_FALLING'               =>  'Fall',
     'MOD_SUICIDE'               =>  'Suicide',
     'MOD_TARGET_LASER'          =>  'Laser',
     'MOD_TRIGGER_HURT'          =>  'Cratered',
    };
}

1;
