package SOF2Logger::Parser::Q3A;

use strict;
use warnings;

use base q/SOF2Logger::Parser/;

use SOF2Logger::ParserBuilder;
use SOF2Logger::Parser::SlotList;
use SOF2Logger::Utils qw/dprint/;

use constant SlotList => 'SOF2Logger::Parser::SlotList';

sub info_from_kill { die "implement me" }
sub is_suicide { die "implement me" }
sub is_headshot_kill { die "implement me" }
sub reset_game_state { warn "implement me" }

# team numbers
sub SPECTATOR_TEAM { 3 }
sub BLUE_TEAM { 2 }
sub RED_TEAM { 1 }

sub change_info_client { 0 }
sub connect_client { 0 }
sub disconnect_client { 0 }

sub shutdown_game {
    my ($self, $time, $info) = @_;
    $self->reset_game_state($time);
}

sub unhandled_line { 0 } # do nothing for unhandled lines

sub get_events
{
  # TODO add a way to track clients that have been kicked
  # 384:11 ClientKick: 7 24.76.139.143:20100 - auto kick for teamkilling
  return { InitGame => 'init_game',
           Kill => 'record_kill',
           ShutdownGame => 'shutdown_game',
           ClientUserinfoChanged => 'change_info_client',
           ClientDisconnect => 'disconnect_client',
           ClientConnect => 'connect_client',
         };
}

sub init_game {
  my ($self, $time, $info) = @_;

  # chop off the leading '\'
  $info = substr($info, 1);

  my %init_info = ( split /\\/, $info );

  my $game_type = $init_info{version};
  my $game_mod = $init_info{gameversion};

  unless ($game_type && exists($init_info{gamename})) {
    # COD uses gamename
    $game_type = $init_info{gamename};
    $game_mod  = $init_info{_Mod};
  }

  no warnings;
  # ask the ParserBuilder to build us a new parser, and set it as
  # our parser
  bless $self, ref(SOF2Logger::ParserBuilder->get_parser($game_type, $game_mod));

  # re-initialize our parser to pick up new info
  $self->init();
  $self->init_game_hook($time, $info);
  $self->reset_game_state();
}

################################################################################
# Sub name: record_kill
# Short: Records a kill from the logfile.
#
# Expects info_from_kill_line(), is_suicide() and is_headshot_kill()
# to be provided by subclass.
#
# Input: $killer_slot, $victim_slot, $kill_method_num, $killer_name, $victim_name, $weapon
# Output: None
################################################################################
sub record_kill
{
    my ($self, $time_in_seconds, $data) = @_;

    my ($killer_id, $victim_id, $weapon_name) =
        eval { $self->info_from_kill_line($data) };

    if ($@) {
        dprint "Not recording kill: $@";
        return;
    }

    unless ( $killer_id && $victim_id && $weapon_name ) {
        dprint("Not enough info to record kill: "
            . "k_id: $killer_id v_id: $victim_id w:$weapon_name line: $.")
                if $self->debug;
        return;
    }

    $self->is_suicide($killer_id, $victim_id, $weapon_name)
    ? SOF2Logger::Kills->add_suicide( $victim_id, $time_in_seconds )
    : SOF2Logger::Kills->add_kill( $killer_id,
                                   $victim_id,
                                   $weapon_name,
                                   $self->is_headshot_kill($data) ? 'head' : 'none',
                                   $time_in_seconds );
}

sub color_map {
  my ($self, $color) = @_;
  return $self->{color_map}{$color} if $color;
}

=item strip_colors PLAYER_NAME %COLOR_MAP

Short: Convert the Q3 color control codes to HTML format

=cut
sub colorize_name
{
    my ($self, $name) = @_;

    # convert symbols to their html counterparts
    $name =~ s/\^[\<\>]//g;
    $name =~ s/\</\&lt\;/g;
    $name =~ s/\>/\&gt\;/g;

    return unless $name;

    # two hats means a hat!!!
    $name =~ s/\^\^/\&#94/g;

    $name =~ s/\^(\S)([^\^]+)/<font color=\"@{[$self->color_map($1)]}\">$2<\/font>/g;

    # Lastly, if we didn't catch it, strip it out
    $name =~ s/\^\S//g;

    return $name;
}

# If we can't find the weapon, we return whatever was passed in
sub weapon_map {
  my ($self, $weapon) = @_;
  return $self->{weapon_map}{$weapon} || $weapon;
}

sub parse_line {
    my ($self, $line) = @_;
    my ($time, $command, $info);

    if ( $line =~ / *(\d+):(\d)(\d?)\s?([^:]+):\s*(.*)/ ) {
        $time    = ( $1 * 60 ) + ( $2 * 10 ) + ( $3 ? $3 : 0 );
        $command = $4;
        $info    = $5;
    }
    $self->last_known_good_time($time);
    return $time, $command, $info;
}

sub get_color_map {
  return
    {
     '0' => "#000000", 		# BLACK
     '1' => "#FF0000", 		# RED
     '2' => "#80FF00", 		# GREEN
     '3' => "#FFFF00", 		# YELLOW
     '4' => "#0000FF", 		# BLUE
     '5' => "#00FFFF", 		# CYAN
     '6' => "#FF00FF", 		# MAGENTA
     '7' => "#FFFFFF", 		# WHITE
     '8' => "#AD19D5", 		# PURPLE
     '9' => "#A7F4F1", 		# CYAN

     'a' => "#48E2DC", 		# Cyan
     'b' => "#FAE3A9", 		# Orange
     'c' => "#E5A7E5", 		# Pink
     'd' => "#DEF7FE", 		# Blue
     'e' => "#D71BBC", 		# Pink
     'f' => "#CFFAFB", 		# Blue
     'g' => "#D1F783", 		# Green
     'h' => "#A91859", 		# Red
     'i' => "#789BCB", 		# Blue
     'j' => "#9BEE44", 		# Green
     'k' => "#FEFCDA", 		# Tan
     'l' => "#EEFBFD", 		# Blue
     'm' => "#DB79DA", 		# Pink
     'n' => "#ACC997", 		# Green
     'o' => "#E9C9CB", 		# Pink
     'p' => "#84D6C0", 		# Cyan
     'q' => "#FAEEF1", 		# Tan
     'r' => "#E5A7E5", 		# Pink
     's' => "#6115A3", 		# Blue
     't' => "#B31164", 		# Red
     'u' => "#846FD0", 		# Blue
     'v' => "#5BE593", 		# Green
     'w' => "#000000", 		# Black
     'x' => "#0C2C4C", 		# Blue
     'y' => "#48E2DC", 		# Blue
     'z' => "#EFD497", 		# Orange
     'A' => "#7918CE", 		# Purple
     'B' => "#9CED8B", 		# Green
     'C' => "#E3AA87", 		# Orange
     'D' => "#889CB8", 		# Blue
     'E' => "#CEEF99", 		# Green
     'F' => "#1F5C9A", 		# Blue
     'G' => "#A8EB95", 		# Green
     'H' => "#D5CCCC", 		# Pink
     'I' => "#C0F0F0", 		# Blue
     'J' => "#F5FCEF", 		# Gray
     'K' => "#F6FEDC", 		# Green
     'L' => "#F1DAF7", 		# Purple
     'M' => "#44E0BA", 		# Cyan
     'N' => "#EBDA6F", 		# Orange
     'O' => "#FFFFFF", 		# White
     'P' => "#35D936", 		# Green
     'Q' => "#F79DF1", 		# Pink
     'R' => "#7686D7", 		# Blue
     'S' => "#9CED8B", 		# Green
     'T' => "#DDAB9F", 		# Orange
     'U' => "#CDA3E5", 		# Purple
     'V' => "#DBFAF8", 		# Gray
     'W' => "#9F25B8", 		# Purple
     'X' => "#6FF2EE", 		# Cyan
     'Y' => "#D4F353", 		# Green
     'Z' => "#F8EEFD", 		# Purple

     '!' => "#000000", 		# BLACK
     '@' => "#F7D43B", 		# ORANGE
     '#' => "#000000", 		# BLACK
     '$' => "#C71819", 		# RED
     '&' => "#A39DC5", 		# BLUE
     '*' => "#F7FCFB", 		# WHITE
     '(' => "#CD4624", 		# PINK
     ')' => "#40E3E7", 		# CYAN
     '_' => "#A82526", 		# RED
     '-' => "#8A8789", 		# GRAY
     '+' => "#FFFFFF", 		# WHITE
     '=' => "#BED4C9", 		# CYAN
     '{' => "#CD4624", 		# RED
     '[' => "#A5EEE1", 		# CYAN
     '}' => "#6E939B", 		# BLUE
     ']' => "#89EEE4", 		# CYAN
     '|' => "#D6F7B5", 		# GREEN
     '\\' => "#D6ABCE", 	# PINK
     ':' => "#FFFFFF", 		# WHITE
     '\'' => "#8A8789", 	# GRAY
     '<' => "#2CE02F", 		# GREEN
     ',' => "#DDE8D6", 		# GRAY
     '>' => "#81F2F1", 		# CYAN
     '.' => "#A39DC5", 		# BLUE
     '?' => "#D6F7B5", 		# GREEN
     '/' => "#411959"  		# PURPLE
    };
}

1;
