package SOF2Logger::Parser::SOF2;

use strict;
use warnings;

use base 'SOF2Logger::Parser::Q3A';

use Carp qw/confess cluck/;

use SOF2Logger::Kills;
use SOF2Logger::Parser::SlotList;
use SOF2Logger::Player;
use SOF2Logger::Utils qw/create_get_set strip_q3_colors/;

use constant SlotList => 'SOF2Logger::Parser::SlotList';
use constant SPECTATOR_TEAM => 3;
use constant Utils => 'SOF2Logger::Utils';

sub WORLD_ID { 'world' };
sub WORLD_SLOT { 1022 };

# NOTES: We should be able to determine several things from the init line:

# who the admin is, and their email:
# \Admin\xTalesx and Admin\Admin E-Mail\Haze@clanwarz.net\

# the server's website:
# \URL\^3http://64.177.245.195/loto/home.htm\

# The specs
# \Server Spec\Dual pentium 4\

# The version of SOF2 they're running: We may be able to use this to
# determine if they are trying to parse the wrong file...
# \version\SOF2MP V1.02a linux-i386 Sep 24 2002\

# TODO add a way to track clients that have been kicked
# 384:11 ClientKick: 7 24.76.139.143:20100 - auto kick for teamkilling

sub get_id {
  my ($self, $name, $slot) = @_;
  my $id_from_name = Utils->clean_q3_color_string($name);
  my $player = SOF2Logger::Player->find($id_from_name);

  if ($slot && $slot->player) {
    return $slot->player->id;
  } elsif ($player) {
    return $player->id;
  } elsif ($id_from_name eq $self->WORLD_ID()) {
    # create the world player
    SOF2Logger::Player->find_or_create( id => $id_from_name, name => $id_from_name );
    return $id_from_name;
  }

  return undef;
}

sub info_from_kill_line {
  my ($self, $data) = @_;

  my ($short_info, $long_info) = split /:/, $data, 2;
  my ($killer_slot, $victim_slot, $weapon_num) = split ' ', $short_info, 3;
  my ($killer_name, $victim_name, $weapon_id)
    = $long_info =~ / (.*?) killed (.*?) by (\w+)/;

  # don't record team change events
  die "Not a kill, a teamchange" if $weapon_id eq 'MOD_TEAMCHANGE';

  # create players for killer and victim, in case they don't exist,
  # since 1.0 doesn't track won-ids.
#  SOF2Logger::Player->find_or_create(id => $killer_id, name => $killer_name);
#  SOF2Logger::Player->find_or_create(id => $victim_id, name => $victim_name);

  return ( $self->get_id($killer_name, SlotList->get_slot($killer_slot)),
	   $self->get_id($victim_name, SlotList->get_slot($victim_slot)),
	   $self->weapon_map($weapon_id) );
}

# TODO: Is there ever a case where killer_id != vic_id, but
# weapon_name = MOD_SUICIDE?
sub is_suicide {
  my ($self, $killer_id, $victim_id, $weapon_name) = @_;
  return ( $killer_id eq $self->WORLD_ID
           || $weapon_name eq 'MOD_SUICIDE'
           || $killer_id eq $victim_id );
}

sub is_headshot_kill { 0 }

sub change_info_client
{
    my ($self, $time_in_seconds, $info) = @_;
    my ( $slot_num, $player_name, $team ) = $info =~ /(\d+) n\\(.+?)\\t\\(\d)/;

# if there's no one there, I'm confused because we didn't get a
# 229:41 ClientConnect: 10 - xxx.xx.xxx.xx:1025 [00000000001111111111111100000000]
#followed by
# 229:41 ClientUserinfoChanged: 10 n\{LLC}miCro\t\2\identity\civilianmale1
    if ( !SlotList->get_slot($slot_num) ) {
      my $config = SOF2Logger::Config->instance();
      cluck("No ClientConnect slot: $. : $info\n") if $config->debug;
      return;
    }

    my $player = SlotList->get_slot($slot_num)->player();
    $player_name =~ s/\t+//g;
    $player->name($player_name);

    SlotList->put_player_in_slot($slot_num,
				 $player,
				 $team,
				 $time_in_seconds,
				 ($team == SPECTATOR_TEAM() ? 1 : 0));
}

sub parse_client_connect_string_1_00 {
  # v1.00
  # 309:41 ClientConnect: 0 - 85.74.129.236 - -=garbage crow=-g

  my ($self, $info) = @_;

  my ( $slot, $ipaddr, $player_name ) =
    $info =~ /^(\d+) - ([^\s]+) - (.*)/;

  return ( $slot, $ipaddr, Utils->clean_q3_color_string($player_name) );
}

sub parse_client_connect_string_default {
  warn "Warning, trying to parse logs from unknown version of SOF2\n";
  warn "Trying v1.02\n";

  return parse_client_connect_string_1_02(@_);
}

sub parse_client_connect_string_1_02 {
  # v1.02... and on up?
  # 229:41 ClientConnect: 10 - xxx.xx.xxx.xx:1025 [00000000001111111111111100000000]

  my ($self, $info) = @_;

  my ( $slot, $ipaddr, $player_id ) =
    $info =~ /^(\d+) - ([\d.]+):-?\d+ \[(\w+)\]/;

  return ($slot, $ipaddr, substr($player_id, -8));
}

use constant PARSE_CLIENT_CONNECT_STRING_METHOD_NAME => 'parse_client_connect_string_';

sub get_correct_parse_method {
  my $self = shift;

  my $versioned_method_name = PARSE_CLIENT_CONNECT_STRING_METHOD_NAME
    . $self->{game_version};

  # we may not have come across an init_game line yet, so we use the default
  return $self->can($versioned_method_name)
    ? $versioned_method_name
      : PARSE_CLIENT_CONNECT_STRING_METHOD_NAME . 'default';
}

sub reset_game_state {
  my ($self, $time) = @_;

  foreach my $slot (SlotList->all_slots()) {
    SlotList->disconnect($slot->num, $time);
  }
}

sub connect_client {
  my ( $self, $time_in_seconds, $info ) = @_;

  my $meth = $self->get_correct_parse_method();
  my ( $slot, $ipaddr, $player_id ) = $self->$meth($info);

  return unless $player_id;

  SlotList->put_player_in_slot($slot,
			       SOF2Logger::Player->find_or_create(id => $player_id),
			       SPECTATOR_TEAM,
			       $time_in_seconds,
			       1);
}

sub init_game_hook {
  my ($self, $time, $info) = @_;
  my %init_info = ( split /\\/, substr($info, 1) );

  my ($game_version) = $init_info{version} =~ /SOF2MP V([\d.]+)/;
  $game_version =~ s/\./_/g;

  $self->{game_version} = $game_version || 'default';
  SOF2Logger::ServerStats->server_admin($self->colorize_name($init_info{Admin}));
  SOF2Logger::ServerStats->server_admin_email($init_info{'Admin E-Mail'});
  SOF2Logger::ServerStats->server_name($self->colorize_name($init_info{sv_hostname})) if $init_info{sv_hostname};
}

sub disconnect_client {
    my ($self, $time, $info) = @_;
    $info =~ /(\d+)/o;
    my $slot = $1;
    SlotList->disconnect($slot, $time)
}

sub get_weapon_map {
  return
    {
     'MOD_AK74_ASSAULT_RIFLE'      =>  'AK74',
     'MOD_AK74_ASSAULT_RIFLE_ALT'  =>  'AK74 Stab',
     'MOD_ANM14_GRENADE'           =>  'Fire Nade',
     'MOD_ANM14_GRENADE_ALT'       =>  'Fire Nade Rolled',
     'MOD_CRUSH'                   =>  'Crushed',
     'MOD_FALLING'                 =>  'Ground Fall',
     'MOD_KNIFE'                   =>  'Knife',
     'MOD_KNIFE_ALT'               =>  'Knife Throw',
     'MOD_M15_GRENADE'             =>  'Smoke Nade',
     'MOD_M15_GRENADE_ALT'         =>  'Smoke Nade Rolled',
     'MOD_M1911A1_PISTOL'          =>  'M1911-A1',
     'MOD_M1911A1_PISTOL_ALT'      =>  'M1911-A1: Head Smash',
     'MOD_M3A1_SUBMACHINEGUN'      =>  'M3A1 SMG',
     'MOD_M4_ASSAULT_RIFLE'        =>  'M4',
     'MOD_M4_ASSAULT_RIFLE_ALT'    =>  'M4 Nade',
     'MOD_M590_SHOTGUN'            =>  'Shottie',
     'MOD_M590_SHOTGUN_ALT'        =>  'Shottie Butt',
     'MOD_M60_MACHINEGUN'          =>  'M60 MG',
     'MOD_M84_GRENADE'             =>  'Flash Nade',
     'MOD_M84_GRENADE_ALT'         =>  'Flash Nade Rolled',
     'MOD_MICRO_UZI_SUBMACHINEGUN' =>  'Micro Uzi',
     'MOD_MM1_GRENADE_LAUNCHER'    =>  'MM1 Grenade Launcher',
     'MOD_MP5'                     =>  'MP5',
     'MOD_MSG90A1_SNIPER_RIFLE'    =>  'Sniper Rifle',
     'MOD_RPG7_LAUNCHER'           =>  'RPG Launcher',
     'MOD_SIG551'                  =>  'SIG551',
     'MOD_SILVER_TALON'            =>  'Silver Talon',
     'MOD_SILVER_TALON_ALT'        =>  'Silver Talon: Head Smash',
     'MOD_SMOHG92_GRENADE'         =>  'Frag Nade',
     'MOD_SMOHG92_GRENADE_ALT'     =>  'Frag Nade Rolled',
     'MOD_SUICIDE'                 =>  'Suicide',
     'MOD_TARGET_LASER'            =>  'OSP Admin Strike',
     'MOD_TEAMCHANGE'              =>  'Team Change',
     'MOD_TELEFRAG'                =>  'TeleFraged',
     'MOD_TRIGGER_HURT'            =>  'Cratered',
     'MOD_TRIGGER_HURT_NOSUICIDE'  =>  'Demo Bomb',
     'MOD_UNKNOWN'                 =>  'Unknown',
     'MOD_US_SOCOM_PISTOL'         =>  'SOCOM',
     'MOD_US_SOCOM_PISTOL_ALT'     =>  'SOCOM: Head Smash',
     'MOD_USAS_12_SHOTGUN'         =>  'Auto Shottie',
     'MOD_WATER'                   =>  'Drowned',
    };
}

1;
