#!/usr/bin/perl

use strict;
use Test::More tests => 54;

use SOF2Logger::Parser::SlotList;
use SOF2Logger::Player;
use SOF2Logger::Kills;
use File::Basename qw/dirname/;

use constant Utils => 'SOF2Logger::Utils';
use constant SlotList => 'SOF2Logger::Parser::SlotList';

BEGIN { use_ok('SOF2Logger::Parser::SOF2') }

# grab the test config file from the config test files
my $config =
  SOF2Logger::Config->instance({config_file => dirname(__FILE__) . '/../../TEST/logparse.conf'});

test_general_cases();
test_weapon_map();
test_change_info_client();
test_shutdown_game();
test_init_game();
test_disconnect_client();
test_record_kill();
test_adds_kills_based_on_id_instead_of_name();
test_version_1_00_parses_client_connect_correctly();

# I can't remember if we decided not to do this or not.. somehow I thought we
# realized that this test was incorrect...
# test_strips_whitespace_characters_from_name();

sub test_record_kill
{
    my $parser = create_test_players_in_slots();
    my $p1_name = 'hottie';
    my $p1_id   = '0000000A';
    my $p2_name = 'billy';
    my $p2_id   = '0000000B';
    my $p3_name = 'bobby';
    my $p3_id   = '0000000C';

    $parser->record_kill(3478, '1 2 8: KOBKED killed scrags by MOD_MP5');
    is(SOF2Logger::Kills->kills_for_player($p1_id),1, 'kills for hottie');
    is(SOF2Logger::Kills->deaths_for_player($p2_id),1, 'deaths for billy');

    $parser->record_kill(3499, '1022 3 29: world killed scrags by MOD_TRIGGER_HURT_NOSUICIDE');
    is(SOF2Logger::Kills->deaths_for_player($p2_id),1, 'demo bomb deaths for billy');
    is(SOF2Logger::Kills->kills_for_player($parser->WORLD_ID),0, 'kills for <world>');

    $parser->record_kill(3499, '1022 3 18: world killed scrags by MOD_SMOHG92_GRENADE');
    is(SOF2Logger::Kills->suicides_for_player($p3_id),2, 'suicides for bobby');

    # bogus slot
    $parser->record_kill(3499, '1 4 18: KOBKED killed scrags by MOD_BOGUS');
    is(SOF2Logger::Kills->kills_for_player($p1_id), 1, 'kills for hottie after bogus slot');

    $parser->record_kill(3499, '4 1 18: KOBKED killed scrags by MOD_BOGUS');
    is(SOF2Logger::Kills->deaths_for_player($p1_id), 0, 'deaths for hottie after bogus slot');
}

sub test_general_cases
{
    my $filename = 'foo';
    my $sof_parser = SOF2Logger::Parser::SOF2->new('logfile.log', 0);
    my $event;
    
    is(ref $sof_parser, 'SOF2Logger::Parser::SOF2', 'isa our class');

    (undef, $event, undef) = $sof_parser->parse_line(' 10:00 Kill:');
    is($event, 'Kill', 'match a kill line');

    (undef, $event, undef) = $sof_parser->parse_line(' Kill:');
    is($event, undef, 'match not a kill line');
    
    (undef, $event, undef) = 
        $sof_parser->parse_line(' 17:36 ClientUserinfoChanged:');
    is($event, 'ClientUserinfoChanged', 'match a client info changed line');

    (undef, $event, undef) = 
        $sof_parser->parse_line(' 17:36 ClientUserinfo:');
    ok($event ne 'ClientUserInfoChanged', 'match a client info changed line');

    (undef, $event, undef) = 
        $sof_parser->parse_line('   18:36 ClientConnect:');
    is($event, 'ClientConnect', 'match client connect line');

    (undef, $event, undef) = 
        $sof_parser->parse_line('   18: ClientConnect:');
    ok(!$event, 'match not client connect line');

    (undef, $event, undef) = 
        $sof_parser->parse_line('  38:23 ShutdownGame:');
    is($event, 'ShutdownGame','match shutdown game line');

    (undef, $event, undef) = 
        $sof_parser->parse_line('  38:23 ShutGameDown:');
    ok($event ne 'ShutdownGame','match not shutdown game line');

    (undef, $event, undef) = 
        $sof_parser->parse_line('  38:23 InitGame:');
    is($event, 'InitGame','match init game');

    (undef, $event, undef) = 
    $sof_parser->parse_line(':23 InitGame:');
    ok(!$event, 'match not init game');

    (undef, $event, undef) = 
        $sof_parser->parse_line(' 42:10  ClientDisconnect');
    ok(!$event,'match not client disconnect');
    
    # test some time stuff
    (undef, $event, undef) = 
      $sof_parser->parse_line('44938:33ShutdownGame:');
    is($event, 'ShutdownGame','match compressed time on shutdown game line');

    is($sof_parser->last_known_good_time, 2696313, 'parse line sets time properly');
    is($sof_parser->weapon_map('MOD_KNIFE'), 'Knife', ' method maker test');
}

sub test_disconnect_client
{
    SOF2Logger::Player->delete_all();
    my $parser = create_test_players_in_slots();
    my $p2_name = 'billy';
    my $p2_id = '0000000B';

    $parser->disconnect_client(1073, '2');

    is(SlotList->get_slot(2), undef, 'after disconnect');
    is(SOF2Logger::Player->find($p2_id)->total_time,120, 'total time for billy');
}

sub test_init_game
{
    my $parser = create_test_players_in_slots();

    $parser->init_game(89969, '\sv_pure\0\scorelimit\12\timelimit\25\sv_floodProtect\0\sv_maxPing\350\sv_minPing\0\sv_maxRate\10000\sv_punkbuster\1\sv_hostname\^1-=rK|RealDamage ^&W/STATS^1=-\dmflags\8\g_needpass\0\sv_maxclients\20\sv_allowAnonymous\1\sv_allowDownload\0\g_maxGameClients\20\g_friendlyfire\1\sv_enableAdminPowers\1\sv_ospClient\1\g_gametype\ctf\version\SOF2MP V1.02a linux-i386 Sep 24 2002\fraglimit\20\protocol\2004\mapname\mp_shop2\.Configured By\SOF2 Runner: February 22 2003 beta 0.9.4\.Online Since\7/19/2003 10:09:32 PM\gamename\sof2mp\Config File\SOF2\Admin\xTalesx and Admin\Admin E-Mail\Haze@clanwarz.net\URL\^3http://64.177.245.195/loto/home.htm\Server Spec\Dual pentium 4\IRC\#SOF2 on irc.enterthegame.com\g_available\22222222022222002222\gameversion\OSP++ v0.1b beta\g_plusplus\1\team_redName\Red team\team_blueName\Blue team');
    my @ref = SlotList->all_slots();

    is($parser->{game_version}, '1_02', 'game version is set correctly by init line');
    is_deeply(\@ref, [], 'after init game');
}

sub test_version_1_00_parses_client_connect_correctly {
  my $parser = create_test_players_in_slots();

  $parser->init_game(295, '\sv_pure\2\g_rpmClientsOnly\0\g_allowThirdPerson\0\g_availableWeapons\22222222222002222\scorelimit\0\g_friendlyfire\2\g_maxGameClients\16\sv_floodProtect\1\sv_maxPing\500\sv_minPing\0\sv_maxRate\15000\sv_maxclients\16\sv_hostname\^bwww.^1SOF^7PLAYERS^b.co.uk\timelimit\15\dmflags\010\g_allowThirdCross\0\g_instaGib\0\g_goldenGun\0\version\SOF2MP V1.00 linux-i386 May  6 2002\fraglimit\20\g_gametype\ctf\g_needpass\0\protocol\2002\mapname\arioche/mp_small\sv_privateClients\0\sv_allowAnonymous\0\sv_allowDownload\0\.Administrator\Pirus\.Email\info@gameage.co.uk\.Hosted By\gameage.co.uk Online Gaming\.URL\http://www.gameage.co.uk\.iRC\irc.quakenet.eu.org / #gameage-uk\.Location\UK\gamename\sof2mp\modname\PRO 1.31');

  is($parser->{game_version}, '1_00', 'game version is correct');

  is($parser->get_correct_parse_method(),
     'parse_client_connect_string_1_00',
     'client connect method name is correct');

  is_deeply([$parser->parse_client_connect_string_1_00('0 - 85.74.129.236 - -=garbage crow=-g')],
            [ '0', '85.74.129.236', 'garbagecrowg'],
            'V1.00 parser parses connect line correctly');
}

sub test_shutdown_game
{
    SOF2Logger::Player->delete_all();
    my $parser = create_test_players_in_slots();

    my $p1_name = 'hottie';
    my $p1_id   = '0000000A';
    my $p2_name = 'billy';
    my $p2_id   = '0000000B';
    my $p3_name = 'bobby';
    my $p3_id   = '0000000C';

    $parser->shutdown_game(1073, '');

    # since hottie was spec-ing, they should have 0 time
    is(SOF2Logger::Player->find($p1_id)->total_time,0, 'total time for hottie');
    is(SOF2Logger::Player->find($p2_id)->total_time,120, 'total time for billy');
    is(SOF2Logger::Player->find($p3_id)->total_time,120, 'total time for bobby');

    my @slots = SlotList->all_slots();
    is(scalar @slots, 0, 'after shutdown game');
}

sub test_change_info_client
{
    my $parser = create_test_players_in_slots();
    my $p1_name = 'hottie';
    my $p1_id   = '0000000A';
    my $p2_name = 'billy';
    my $p2_id   = '0000000B';
    my $p3_name = 'bobby';
    my $p3_id   = '0000000C';

    my $slot = SlotList->get_slot(1);

    is($slot->team(),3, 'player hotties team');
    is(SOF2Logger::Player->find($p2_id)->total_time,0, 'player billys time');
    is(!SlotList->get_slot(3)->is_spectator(), 1, 'bobby is playing');
    my @is = SlotList->all_slots;
    my $should_be = [
          {
            'IS_SPECTATOR' => 1,
            'IS_PLAYING' => 1,
            'PLAYER' => bless( {
                                 'name' => 'hottie',
                                 'SOF2Logger::Player__boolean' => ' ',
                                 'flag_caps' => 0,
                                 'total_time' => 0
                               }, 'SOF2Logger::Player' ),
            'TEAM' => '3'
          },
          {
            'IS_SPECTATOR' => 0,
            'IS_PLAYING' => 1,
            'PLAYER' => bless( {
                                 'name' => 'billy',
                                 'SOF2Logger::Player__boolean' => ' ',
                                 'flag_caps' => 0,
                                 'total_time' => 0
                               }, 'SOF2Logger::Player' ),
            'TEAM' => '2',
            'START_TIME' => '953'
          },
          {
            'IS_SPECTATOR' => 0,
            'IS_PLAYING' => 1,
            'PLAYER' => bless( {
                                 'name' => 'bobby',
                                 'SOF2Logger::Player__boolean' => ' ',
                                 'flag_caps' => 0,
                                 'total_time' => 0
                               }, 'SOF2Logger::Player' ),
            'TEAM' => '1',
            'START_TIME' => '953'
          }
        ];

    is(eq_array(\@is, $should_be),0, 'what is, is what should be');

    is($slot->player()->name(), $p1_name, 'player at slot 1 name');
    # billy moving to spectator
    $parser->change_info_client(1022, '2 n\billy\t\3\identity\marinesoldier1\sir\0\sib\0\grade\3');
    is(SlotList->get_slot(2)->is_spectator(), 1, 'after spec change');
    is(SlotList->get_slot(2)->player()->total_time, 1022 - 953, 'total time is correct');

    $parser->disconnect_client(1022, '2');

    $parser->connect_client(1022, "2 - 192.168.0.31:1025 [F000000000111111111111110000000D]");
    $parser->change_info_client(
        1022,
        '2 n\hooligan\t\3\identity\marinesoldier1\sir\0\sib\0\grade\3');

    is(SlotList->get_slot(2)->player()->name, 'hooligan', 'name change is correct');
    is(SlotList->get_slot(2)->player()->total_time, 0, 'and resets time for new name');
    is(SlotList->get_slot(2)->is_spectator(), 1, 'and doesnt affect spectator');
    is(SOF2Logger::Player->find($p2_id)->total_time, 69, 'and doesnt affect time for billy');

    $parser->change_info_client(
        1030,
        '1 n\doofus\t\1\identity\marinesoldier1\sir\0\sib\0\grade\3');
    is(SlotList->get_slot(1)->is_spectator(), 0, 'doofus is not a spectator');
    is(SlotList->get_slot(1)->player()->name, 'doofus', 'doofus is doofus');

    $parser->change_info_client(
        1026,
        '2 n\hooligan\t\2\identity\marinesoldier1\sir\0\sib\0\grade\3');
    is(SlotList->get_slot(2)->is_spectator, 0, 'hooligan is not spectating');
    is(SlotList->get_slot(2)->team, 2, 'hooligan has changed teams');

    $parser->change_info_client(
        1045,
        '2 n\hooligan\t\3\identity\marinesoldier1\sir\0\sib\0\grade\3');
    is(SlotList->get_slot(2)->is_spectator, 1, 'hooligan is now spectating');
    is(SlotList->get_slot(2)->player->total_time, 25 - 6, 'and time played is correct');
}

sub test_adds_kills_based_on_id_instead_of_name
{
  SOF2Logger::Kills->delete_all();
    my $name1 = '2 n\Name^7With{bad}^4^sStuff\t\2\identity\marinesoldier1\sir\0\sib\0\grade\0';
    my $victim = '1 n\scrags\t\2\identity\marinesoldier1\sir\0\sib\0\grade\0';
    my $name2 = '2 n\^3^4^5Name^7With{bad}^4^sStuff\t\2\identity\marinesoldier1\sir\0\sib\0\grade\0';
    my $real_name = uc(Utils->clean_q3_color_string('Name^7With{bad}^4^sStuff'));
    my $name1_id = '0000000A';
    my $test_parser = SOF2Logger::Parser::SOF2->new('logfile.log', 0);
    $test_parser->{game_version} = '1_02';

    $test_parser->connect_client(933, "2 - 192.168.0.25:1025 [F000000000111111111111110000000A]");
    $test_parser->connect_client(934, "1 - 192.168.1.25:1025 [F000000000111111111111110000000B]");
    $test_parser->change_info_client(933, $name1);
    $test_parser->change_info_client(934, $victim);
    $test_parser->debug(1);
    $test_parser->record_kill(3478, 
        '2 1 8: Name^7With{bad}^4^sStuff killed scrags by MOD_MP5');

    is(SOF2Logger::Kills->kills_for_player($name1_id), 1, 'Kills for player id');

    $test_parser->change_info_client(933, $name2);
    $test_parser->record_kill(3478, 
        '2 1 8: ^3^4^5Name^7With{bad}^4^sStuff killed scrags by MOD_MP5');

    is(SOF2Logger::Kills->kills_for_player($name1_id), 2, 'Kills for player id');
}

# create a test parser with 3 random players on different teams
sub create_test_players_in_slots
{
    my $p1 = '1 n\hottie\t\3\identity\marinesoldier1\sir\0\sib\0\grade\0';
    my $p2 = '2 n\billy\t\2\identity\marinesoldier1\sir\0\sib\0\grade\0';
    my $p3 = '3 n\bobby\t\1\identity\marinesoldier1\sir\0\sib\0\grade\0';
    my $test_parser = SOF2Logger::Parser::SOF2->new('logfile.log', 0);
    $test_parser->{game_version} = '1_02';

    $test_parser->connect_client(930, "1 - 192.168.0.25:1025 [F000000000111111111111110000000A]");
    $test_parser->connect_client(930, "2 - 192.168.0.24:1025 [E000000000111111111111110000000B]");
    $test_parser->connect_client(930, "3 - 192.168.0.23:1025 [D000000000111111111111110000000C]");

    $test_parser->change_info_client(933, $p1);
    $test_parser->change_info_client(953, $p2);
    $test_parser->change_info_client(953, $p3);

    return $test_parser;
}

sub test_weapon_map
{
    my $sof_parser = SOF2Logger::Parser::SOF2->new('logfile.log', 0);
    my $wm = $sof_parser->{weapon_map};
    my $ref = {
        'MOD_M4_ASSAULT_RIFLE_ALT' => 'M4 Nade',
        'MOD_M4_ASSAULT_RIFLE' => 'M4',
        'MOD_M84_GRENADE' => 'Flash Nade',
        'MOD_WATER' => 'Drowned',
        'MOD_TELEFRAG' => 'TeleFraged',
        'MOD_SMOHG92_GRENADE' => 'Frag Nade',
        'MOD_MM1_GRENADE_LAUNCHER' => 'MM1 Grenade Launcher',
        'MOD_CRUSH' => 'Crushed',
        'MOD_ANM14_GRENADE' => 'Fire Nade',
        'MOD_TRIGGER_HURT' => 'Cratered',
        'MOD_SILVER_TALON' => 'Silver Talon',
        'MOD_M1911A1_PISTOL_ALT' => 'M1911-A1: Head Smash',
        'MOD_M1911A1_PISTOL' => 'M1911-A1',
        'MOD_FALLING' => 'Ground Fall',
        'MOD_MSG90A1_SNIPER_RIFLE' => 'Sniper Rifle',
        'MOD_UNKNOWN' => 'Unknown',
        'MOD_M84_GRENADE_ALT' => 'Flash Nade Rolled',
        'MOD_AK74_ASSAULT_RIFLE' => 'AK74',
        'MOD_M60_MACHINEGUN' => 'M60 MG',
        'MOD_KNIFE' => 'Knife',
        'MOD_TARGET_LASER' => 'OSP Admin Strike',
        'MOD_SIG551' => 'SIG551',
        'MOD_RPG7_LAUNCHER' => 'RPG Launcher',
        'MOD_SILVER_TALON_ALT' => 'Silver Talon: Head Smash',
        'MOD_MP5' => 'MP5',
        'MOD_US_SOCOM_PISTOL' => 'SOCOM',
        'MOD_M15_GRENADE' => 'Smoke Nade',
        'MOD_AK74_ASSAULT_RIFLE_ALT' => 'AK74 Stab',
        'MOD_M590_SHOTGUN_ALT' => 'Shottie Butt',
        'MOD_SMOHG92_GRENADE_ALT' => 'Frag Nade Rolled',
        'MOD_USAS_12_SHOTGUN' => 'Auto Shottie',
        'MOD_US_SOCOM_PISTOL_ALT' => 'SOCOM: Head Smash',
        'MOD_TEAMCHANGE' => 'Team Change',
        'MOD_KNIFE_ALT' => 'Knife Throw',
        'MOD_TRIGGER_HURT_NOSUICIDE' => 'Demo Bomb',
        'MOD_M15_GRENADE_ALT' => 'Smoke Nade Rolled',
        'MOD_SUICIDE' => 'Suicide',
        'MOD_MICRO_UZI_SUBMACHINEGUN' => 'Micro Uzi',
        'MOD_M590_SHOTGUN' => 'Shottie',
        'MOD_M3A1_SUBMACHINEGUN' => 'M3A1 SMG',
        'MOD_ANM14_GRENADE_ALT' => 'Fire Nade Rolled'
        };

    is_deeply($wm, $ref, 'weapon map test');
}

sub test_strips_whitespace_characters_from_name
{
    my $player_name = "Bad Na\tme  ";
    my $test_parser = SOF2Logger::Parser::SOF2->new('logfile.log', 0);
    $test_parser->{game_version} = '1_02';

    my $p = qq/2 n\\$player_name\\t\\2\\identity\\marinesoldier1\\sir\\0\\sib\\0\\grade\\0/;
    my $test_parser = SOF2Logger::Parser::SOF2->new('logfile.log', 0);

    $test_parser->connect_client(933, "2 - 192.168.0.31:1025 [F000000000111111111111110000000D]");
    $test_parser->change_info_client(933, $p);

    is((($test_parser->game_slots)->[2])->{PLAYER}->name, 'BadName', 'test strips whitespace characters');
}
