package SOF2Logger::Player;

use strict;

use SOF2Logger::Config;
use Carp qw/cluck/;

=head1 NAME

Player.pm

=head1 DESCRIPTION

making player.pm will allow me to encapsulate player aliasing.
Also, this will allow for a better interface into the list of
players.

also, you probably don't want to create an instance of
SOF2Logger::Player, as it only returns PlayerInst objects, which you
actually care about.

=head2 PUBLIC INTERFACE

=over 8

=cut

# class variable to store the list of players;  We should only have one list of players.
my %players;

=item find ($id)

finds a player by 'id'.  This method will do aliasing as well, so if a
player is aliased, the aliased id is substituted instead of the id
passed in.

=cut
sub find {
  my ($class_or_self, $id) = @_;

  # do player aliasing
  $id = SOF2Logger::Config->instance()->player_alias($id);

  return $players{$id};
}

=item find_or_create ( id => '', total_time => 0 )

attempts to find a player by id, and update info, and if not, creates one with the
information provided

=cut
sub find_or_create {
  my ($class_or_self, %options) = @_;

  unless ($options{id}) {
    cluck "id is required paramter to find_or_create\n";
    return;
  }

  my $player;

  # if we can't find it, go ahead and create it
  unless ($player = $class_or_self->find($options{id})) {
    $player = $class_or_self->create(%options);
  }

  # update player info
  for my $option (keys %options) {
    next if $option eq 'id';
    $player->$option($options{$option})
      if $player->can($option);
  }

  return $player;
}

=item find_all

returns a list of all the players.

=cut
sub find_all {
  my $self = shift;
  return values %players;
}

=item create ({ id => '', total_time => 0, avatar_name => 'frank.jpg' })

returns a ref to the newly created player, this means that you won't
be able to call "find" on the returned player object.

Also, this method will perform player aliasing, so it will substitute
the aliased id if appropriate

=cut
sub create {
  my ($class_or_self, %options) = @_;

  unless ($options{id}) {
    warn "id is required parameter to create\n";
    return;
  }

  # do player aliasing
  $options{id} = SOF2Logger::Config->instance()->player_alias($options{id});

  my $player = SOF2Logger::Player::PlayerInst->new(\%options);
  $players{$options{id}} = $player;
  return $player;
}

=item delete ($id)

deletes a player with id $id

=cut
sub delete {
  my ($class, $id) = @_;
  $id = SOF2Logger::Config->instance()->player_alias($id);
  delete $players{$id};
}

=item delete_all

deletes all the players

=cut
sub delete_all {
  %players = ();
}

# Inner player package

{
  package SOF2Logger::Player::PlayerInst;

  use SOF2Logger::Utils qw/create_get_set/;

  sub new {
    my ($class, $options) = @_;
    my $self = {};
    bless $self, $class;

    die "you cannot create a player without an id" unless $options->{id};

    $self->{total_time} = 0;
    $self->{flag_caps} = 0;
    $self->{flag_returns} = 0;

    # rating defaults to 1400
    $self->{rating} = 1400;

    my @members = qw/name total_time flag_caps flag_returns key id avatar_name is_filtered rating/;
    for my $member (@members) {
      create_get_set($class, $member);
      $self->$member($options->{$member}) if $options->{$member};
    }

    return $self;
  }

  sub add_time {
    my ($self, $time) = @_;
    my $new_time = $self->total_time + $time;
    $self->total_time($new_time);
  }

  sub add_cap {
    my ($self) = @_;
    # print ( $self->name . " captured the flag\n");
    $self->flag_caps($self->flag_caps + 1);
  }

  sub add_return {
    my ($self) = @_;
    # print ( $self->name . " returned the flag\n");
    $self->flag_returns($self->flag_returns + 1);
  }

  sub delete {
    my ($self) = @_;
    SOF2Logger::Player->delete($self->id);
    undef $self;
  }
}

sub freeze {
  my ($class, $file) = @_;
  open(FILE, '>', $file)
    or die "Err: could not freeze to file $file: $!\n";

  foreach my $player ($class->find_all()) {
    print FILE join( "\t",
                     $player->id,
                     $player->name,
                     $player->total_time,
                     $player->flag_caps,
                     $player->flag_returns,
                     $player->avatar_name,
                     $player->rating );
    print FILE "\n";
  }

  close(FILE);
}

sub thaw {
  my ($class, $file) = @_;
  return unless -f $file;
  open(FILE, $file)
    or die "Err: could not thaw from file $file: $!\n";

  while (my $line = <FILE>) {
    chomp $line;
    my ($id, $name, $total_time, $flag_caps, $flag_returns, $avatar_name, $rating)
      = split /\t/, $line;
    $class->create( id => $id,
                    name => $name,
                    total_time => $total_time,
                    flag_caps => $flag_caps,
                    flag_returns => $flag_returns,
                    avatar_name => $avatar_name,
                    rating => $rating );
  }

  close(FILE);
}

1;
