package SOF2Logger::Reports::IndividualPlayer;

use strict;
use warnings;

use Class::MethodMaker
  new_with_init => 'new',
  get_set       => [qw/config player kills dataset/],
  hash          => [qw/players/];

use SOF2Logger::Config;
use SOF2Logger::Kills;
use SOF2Logger::Player;
use SOF2Logger::OutputHTML qw/strip_colors/;
use SOF2Logger::Utils;
use Template;

use constant TEMPLATE_FILE => 'IndividualPlayer.html';

=head1 NAME

IndividualPlayer.pm : the report for an individual player

=head1 DESCRIPTION

Renders an html page with all the individual player stats that we track

=head2 PUBLIC INTERFACE

=over 8

=cut

=item new PLAYER ALL_PLAYERS
	
Creates a new object from the class.
PLAYER is the player you wish to render the report for and ALL_PLAYERS
is a reference to a hash containing all the players seen so far, keyed
off their id.  In the future, ALL_PLAYERS will be removed in favor of 
a table object or some kind of singleton.

=cut

sub init
{
    my ( $self, $player, $players ) = @_;
    $self->config( SOF2Logger::Config->instance );
    $self->player($player);
    $self->kills( SOF2Logger::Kills->instance );
    $self->players($players);
    $self->dataset( $self->_get_dataset );
}

=item render

Render the report and return the string containing the rendered report, which is
a .html string.  This should be used to write out to a file, but could be used in
a CGI environment.

=cut

sub render
{

# TODO make sure Template toolkits works outside of current project directories.
    my ( $self, $template_path ) = @_;

    my $template_config = {
        INCLUDE_PATH => $template_path,
        EVAL_PERL    => 1,
    };

    my $report_str = '';

    my $template = Template->new($template_config);
    $template->process( TEMPLATE_FILE, $self->dataset, \$report_str )
      or die $template->error();

    return $report_str;
}

sub _get_dataset
{
    my ($self) = @_;
    my $dataset;
    $dataset->{summary}       = $self->_get_summary_dataset;
    $dataset->{fav_weapons}   = $self->_get_fav_weapons_dataset;
    $dataset->{fav_targets}   = $self->_get_fav_targets_dataset;
    $dataset->{worst_enemies} = $self->_get_worst_enemies;
    $dataset->{row_sub}       = sub {
        my ($count) = shift;
        my @blocks = ( 2, 1, 1, 2 );
        return 'block' . $blocks[ ( $count % 4 ) ];
    };
    $dataset->{title} = SOF2Logger::Utils->strip_q3_colors( $self->player->name );

    return $dataset;
}

sub _get_summary_dataset
{

# TODO Skill should be calculated as a percentage of all other players ie. you are better then x% of players,
# comparing their raw skill number to your raw skill number
    my ($self)    = @_;
    my $kills     = $self->kills->kills_for_player( $self->player->id );
    my $deaths    = $self->kills->deaths_for_player( $self->player->id );
    my $suicides  = $self->kills->suicides_for_player( $self->player->id );
    my $headshots = $self->kills->headshots_for_player( $self->player->id );
    my $skill     = $kills - $deaths - $suicides;
    my $time      = $self->player->total_time;
    my $ratio = int( ( $kills / (( $kills + $deaths + $suicides ) || 1) ) * 100 );

    return {
        name      => strip_colors( $self->player->name ),
        kills     => $kills,
        deaths    => $deaths,
        suicides  => $suicides,
        headshots => $headshots,
        skill     => $skill,
        time      => $time,
        ratio     => $ratio,
    };
}

sub _get_fav_weapons_dataset
{
    my ($self)      = @_;
    my $player      = $self->player;
    my $weapon_info = $self->kills->kills_by_weapon_for_player( $player->id );
    my $total_kills = $self->kills->kills_for_player( $player->id );

    my $dataset;

    for my $info (@$weapon_info)
    {
        my ( $weapon, $times_used ) = @$info;
        push @$dataset,
          {
            name  => $weapon,
            kills => $times_used,
            ratio => int( ( $times_used / $total_kills ) * 100 ),
          };
    }
    return $dataset;
}

sub _get_fav_targets_dataset
{
    my ($self) = @_;

    my $victim_info =
      $self->kills->kills_by_player_for_player( $self->player->id );
    my $total_kills = $self->kills->kills_for_player( $self->player->id );
    my $limit       = 25;
    my $count       = 1;
    my $dataset;

    foreach my $info (@$victim_info)
    {
        my ( $vic_id, $deaths ) = @$info;
        last if $count > $limit;
        my $victim = $self->players_exists($vic_id) && $self->players($vic_id);
        next unless $victim;
        $count++;

        push @$dataset,
          {
            name  => ( $self->_link_player_to_filename( $victim ) ),
            kills => $deaths,
            ratio => int( ( $deaths / $total_kills ) * 100 ),
          };
    }
    return $dataset;
}

sub _get_worst_enemies
{
    my ($self) = @_;

    my $dataset;
    my $killer_info =
      $self->kills->deaths_by_player_for_player( $self->player->id );
    my $total_deaths = $self->kills->deaths_for_player( $self->player->id );
    my $limit        = 25;
    my $count        = 1;

    foreach my $info (@$killer_info)
    {
        my ( $killer_id, $kills ) = @$info;
        last if $count > $limit;
        my $killer =
          $self->players_exists($killer_id) && $self->players($killer_id);
        next unless $killer;

        $count++;
        push @$dataset,
          {
            name  => $self->_link_player_to_filename( $killer ),
            kills => $kills,
            ratio => int( ( $kills / $total_deaths ) * 100 ),
          };
    }
    return $dataset;
}

sub _link_player_to_filename
{
    my ($self, $player) = @_;
    #my $player_dir = $self->config->player_data_dir;

    return $player->is_filtered
        ? strip_colors( $player->name )
        : sprintf qq(<a href="%s.html" class="player">%s</a>),
            $player->id, strip_colors($player->name);
            #$player_dir . "/" . $player->id, strip_colors($player->name);
}

1;
