#!/usr/bin/perl

use warnings;
use strict;
use Test::More skip_all => 'not implemented';

use Cwd;
use SOF2Logger::Reports::IndividualPlayer;
use SOF2Logger::Player;
use SOF2Logger::Kills;
use SOF2Logger::Config;

my $config = SOF2Logger::Config->instance();

$config->player_data_dir('players');

test_fav_weapons_dataset();
test_summary_dataset();
test_fav_targets_dataset();
test_worst_enemies_dataset();
test_rendered_report();

sub test_summary_dataset
{
    SOF2Logger::Kills->destroy();
    my $players;
    my $player = SOF2Logger::Player->new( 'Bobbie', 123459 );
    $player->total_time(25);
    $players->{ $player->id } = $player;
    my $victim = SOF2Logger::Player->new( 'Victim', 98765 );
    $players->{ $victim->id } = $victim;
    my $kills = SOF2Logger::Kills->instance();
    $kills->add_kill( $player->id, $victim->id, 'AK',  '', 123 );
    $kills->add_kill( $player->id, $victim->id, 'RPG', '', 130 );
    $kills->add_suicide( $player->id );

    my $report =
      SOF2Logger::Reports::IndividualPlayer->new( $player, $players );

    is_deeply(
        $report->dataset->{summary},
        {
            name      => 'Bobbie',
            kills     => 2,
            deaths    => 0,
            suicides  => 1,
            headshots => 0,
            skill     => 1,
            time      => 25,
            ratio     => int( ( 2 / ( 2 + 0 + 1 ) ) * 100 ),
        },
        'summary dataset'
    );
}

sub test_fav_weapons_dataset
{
    SOF2Logger::Kills->destroy();
    my $players = {};
    my $player  = new SOF2Logger::Player( 'bob', 12345 );
    my $victim  = new SOF2Logger::Player( 'the vic!', 67890 );

    my $kills = SOF2Logger::Kills->instance;
    $kills->add_kill( $player->id, $victim->id, 'shotgun',    'head', 60 );
    $kills->add_kill( $player->id, $victim->id, 'shotgun',    'head', 65 );
    $kills->add_kill( $player->id, $victim->id, 'shotgun',    'head', 66 );
    $kills->add_kill( $player->id, $victim->id, 'machinegun', 'head', 67 );
    $kills->add_kill( $player->id, $victim->id, 'knife',      'head', 70 );
    $kills->add_kill( $player->id, $victim->id, 'machinegun', 'head', 75 );

    my $report =
      SOF2Logger::Reports::IndividualPlayer->new( $player, $players );

    is_deeply(
        $report->dataset->{fav_weapons},
        [
            {
                name  => 'shotgun',
                kills => 3,
                ratio => int( ( 3 / 6 ) * 100 ),
            },
            {
                name  => 'machinegun',
                kills => 2,
                ratio => int( ( 2 / 6 ) * 100 ),
            },
            {
                name  => 'knife',
                kills => 1,
                ratio => int( ( 1 / 6 ) * 100 ),
            },
        ],
        'favorite weapons dataset'
    );
}

sub test_fav_targets_dataset
{
    SOF2Logger::Kills->destroy();
    my $kills   = SOF2Logger::Kills->instance();
    my $players = {};
    my $player  = new SOF2Logger::Player( 'bob', 12345 );
    my $victim  = new SOF2Logger::Player( 'the vic!', 67890 );
    $players->{ $victim->id } = $victim;
    my $victim2 = new SOF2Logger::Player( 'henry', 98765 );
    $players->{ $victim2->id } = $victim2;
    $victim2->is_filtered(1);

    $kills->add_kill( $player->id, $victim->id,  'shotgun',    'head', 60 );
    $kills->add_kill( $player->id, $victim->id,  'shotgun',    'head', 65 );
    $kills->add_kill( $player->id, $victim->id,  'shotgun',    'head', 66 );
    $kills->add_kill( $player->id, $victim->id,  'machinegun', 'head', 67 );
    $kills->add_kill( $player->id, $victim2->id, 'knife',      'head', 70 );
    $kills->add_kill( $player->id, $victim2->id, 'machinegun', 'head', 75 );

    my $report =
      SOF2Logger::Reports::IndividualPlayer->new( $player, $players );

    is_deeply(
        $report->dataset->{fav_targets},
        [
            {
                name  => '<a href="67890.html" class="player">the vic!</a>',
                kills => 4,
                ratio => int( ( 4 / 6 ) * 100 ),
            },
            {
                name  => 'henry',
                kills => 2,
                ratio => int( ( 2 / 6 ) * 100 ),
            },
        ],
        'favorite targets dataset'
    );
}

sub test_worst_enemies_dataset
{
    SOF2Logger::Kills->destroy();
    my $kills   = SOF2Logger::Kills->instance();
    my $players = {};
    my $player  = new SOF2Logger::Player( 'bob', 12345 );
    my $killer  = new SOF2Logger::Player( 'da killa!', 67890 );
    $players->{ $killer->id } = $killer;
    my $killer2 = new SOF2Logger::Player( 'henry', 98765 );
    $players->{ $killer2->id } = $killer2;

    $kills->add_kill( $killer->id,  $player->id, 'shotgun',    'head', 60 );
    $kills->add_kill( $killer->id,  $player->id, 'shotgun',    'head', 65 );
    $kills->add_kill( $killer->id,  $player->id, 'shotgun',    'head', 66 );
    $kills->add_kill( $killer->id,  $player->id, 'machinegun', 'head', 67 );
    $kills->add_kill( $killer2->id, $player->id, 'knife',      'head', 70 );
    $kills->add_kill( $killer2->id, $player->id, 'machinegun', 'head', 75 );

    my $report =
      SOF2Logger::Reports::IndividualPlayer->new( $player, $players );

    is_deeply(
        $report->dataset->{worst_enemies},
        [
            {
                name  => '<a href="67890.html" class="player">da killa!</a>',
                kills => 4,
                ratio => int( ( 4 / 6 ) * 100 ),
            },
            {
                name  => '<a href="98765.html" class="player">henry</a>',
                kills => 2,
                ratio => int( ( 2 / 6 ) * 100 ),
            },
        ],
        'worst enemies dataset'
    );
}

sub test_rendered_report
{
    SOF2Logger::Kills->destroy();
    my $players;
    my $player = SOF2Logger::Player->new( 'Bobbie', 123459 );
    $player->total_time(25);
    $players->{ $player->id } = $player;
    my $victim = SOF2Logger::Player->new( 'Victim', 98765 );
    $players->{ $victim->id } = $victim;
    my $kills = SOF2Logger::Kills->instance();
    $kills->add_kill( $player->id, $victim->id, 'AK',    '', 123 );
    $kills->add_kill( $player->id, $victim->id, 'RPG',   '', 130 );
    $kills->add_kill( $player->id, $victim->id, 'knife', '', 130 );

    $kills->add_suicide( $player->id );
    my $killer = new SOF2Logger::Player( 'da killa!', 67890 );
    $players->{ $killer->id } = $killer;
    my $killer2 = new SOF2Logger::Player( 'henry', 98765 );
    $players->{ $killer2->id } = $killer2;
    $kills->add_kill( $killer->id,  $player->id, 'shotgun',    'head', 60 );
    $kills->add_kill( $killer->id,  $player->id, 'shotgun',    'head', 65 );
    $kills->add_kill( $killer->id,  $player->id, 'shotgun',    'head', 66 );
    $kills->add_kill( $killer->id,  $player->id, 'machinegun', 'head', 67 );
    $kills->add_kill( $killer2->id, $player->id, 'knife',      'head', 70 );
    $kills->add_kill( $killer2->id, $player->id, 'machinegun', 'head', 75 );

    my $report =
      SOF2Logger::Reports::IndividualPlayer->new( $player, $players );

    my $output_str = $report->render(cwd());

    my $rendered_report = q|
<table class="eghtpct">
   <tr>
     <th class="header" colspan="7"><h1>Bobbie</h1></th>
   </tr>
   <tr>
     <th class="rowheader">Kills</th>
     <th class="rowheader">Deaths</th>
     <th class="rowheader">Suicides</th>
     <th class="rowheader">HeadShots</th>
     <th class="rowheader">Skill</th>
     <th class="rowheader">Time</th>
     <th class="rowheader">Ratio</th>
   </tr>
   <tr class="block1">
       <td>3</td>
 	  <td>6</td>
 	  <td>1</td>
 	  <td>0</td>
 	  <td>-4</td>
 	  <td>
 	  	  0h 00m 25s
 	  </td>
 	  <td>
 	  <img src="../ig/left.gif" height="10" alt="left"><img src="../ig/main.gif" height="10" width="30" alt="main"><img src="../ig/right.gif" height="10" alt="right">&nbsp;30%
 	  </td>
   </tr>
 <tr class="tablefooter">
 <td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td>
 </tr>
 </table>
 &nbsp;
 <table class="eghtpct">
   <tr>
     <th class="header" colspan="4">Favorite Weapons</th>
   </tr>
   <tr>
     <th class="rowheader">&nbsp;</th>
     <th class="rowheader">Weapon Name</th>
     <th class="rowheader">Kills</th>
     <th class="rowheader">Ratio</th>
   </tr>
   
   <tr class="block1">
     <td class="rank">1</td>
     <td>knife</td>
     <td>1</td>
     <td>
       <img src="../ig/left.gif" height="10" alt="left"><img src="../ig/main.gif" height="10" width="33" alt="main"><img src="../ig/right.gif" height="10" alt="right">&nbsp;33 %
     </td>
   </tr>
   
   <tr class="block1">
     <td class="rank">2</td>
     <td>RPG</td>
     <td>1</td>
     <td>
       <img src="../ig/left.gif" height="10" alt="left"><img src="../ig/main.gif" height="10" width="33" alt="main"><img src="../ig/right.gif" height="10" alt="right">&nbsp;33 %
     </td>
   </tr>
   
   <tr class="block2">
     <td class="rank">3</td>
     <td>AK</td>
     <td>1</td>
     <td>
       <img src="../ig/left.gif" height="10" alt="left"><img src="../ig/main.gif" height="10" width="33" alt="main"><img src="../ig/right.gif" height="10" alt="right">&nbsp;33 %
     </td>
   </tr>
   
   <tr class="tablefooter">
 	<td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td>
   </tr>
 </table>
 &nbsp;
 <table class="eghtpct">
   <tr>
     <th class="header" colspan="4">Favorite Targets</th>
   </tr>
   <tr>
     <th class="rowheader">&nbsp;</th>
     <th class="rowheader">Victim</th>
     <th class="rowheader">Kills</th>
     <th class="rowheader">Ratio</th>
   </tr>
   
   <tr class="block1">
     <td class="rank">1</td>
     <td>henry</td>
     <td>3</td>
     <td>
       <img src="../ig/left.gif" height="10" alt="left"><img src="../ig/main.gif" height="10" width="100" alt="main"><img src="../ig/right.gif" height="10" alt="right">&nbsp;100 %
     </td>
   </tr>
   
   <tr class="tablefooter">
 	<td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td>
   </tr>
 </table>
 &nbsp;
 <table class="eghtpct">
   <tr>
     <th class="header" colspan="4">Worst Enemies</th>
   </tr>
   <tr>
     <th class="rowheader">&nbsp;</th>
     <th class="rowheader">Enemy</th>
     <th class="rowheader">Deaths</th>
     <th class="rowheader">Ratio</th>
   </tr>
   
   <tr class="block1">
     <td class="rank">1</td>
     <td>da killa!</td>
     <td>4</td>
     <td>
       <img src="../ig/left.gif" height="10" alt="left"><img src="../ig/main.gif" height="10" width="66" alt="main"><img src="../ig/right.gif" height="10" alt="right">&nbsp;66 %
     </td>
   </tr>
   
   <tr class="block1">
     <td class="rank">2</td>
     <td>henry</td>
     <td>2</td>
     <td>
       <img src="../ig/left.gif" height="10" alt="left"><img src="../ig/main.gif" height="10" width="33" alt="main"><img src="../ig/right.gif" height="10" alt="right">&nbsp;33 %
     </td>
   </tr>
   
   <tr class="tablefooter">
 	<td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td><td>&nbsp;</td>
   </tr>
 </table>
	
	|;

    $rendered_report =~ s/\s+//g;
    $output_str      =~ s/\s+//g;
    like( $output_str, qr/$rendered_report/, 'rendered report all html' );
}
