#!/usr/bin/perl -w

use strict;
use Test::More tests => 20;

use SOF2Logger::ServerStats;
use SOF2Logger::Kills;

BEGIN { use_ok('SOF2Logger::Parser') }

my $log = open_log();
my $log_stats = 'log_stats.dat';
my $logparser = SOF2Logger::Parser->new($log, 0);

is($logparser->log, $log, 'logparser filename matches generated filename');

eval {$logparser->parse_log};
is($@, "unhandled line: '0'\n", 'parse_log dies appropriately');

# make sure unhandled line dies approriately
eval {$logparser->unhandled_line('bad line')};
is($@, "unhandled line: 'bad line'\n", 'default unhandled line behaviour');


test_virtual_methods_die();
test_total_time();
test_log_stats($log);


sub test_virtual_methods_die
{
    my @methods = qw/
        record_kill
        change_info_client
        shutdown_game
        init_game
        disconnect_client
        connect_client/;

    foreach my $method (@methods) {
        eval { $logparser->$method };
        is($@, "implement SOF2Logger::Parser::$method\n", "Method $method is virtual");
    }
}

sub test_total_time
{
    my $temp_parser = SOF2Logger::Parser->new('log', 0);
    my $test_player = SOF2Logger::Player->create(name => 'foo', id => 'FOO');
    my $test_player2 = SOF2Logger::Player->create(name => 'foo2', id => 'FOO2');
    SOF2Logger::ServerStats->total_time(50);
    $test_player->add_time(25);
    $test_player->add_time(15);

    $temp_parser->set_total_time;

    is(SOF2Logger::ServerStats->total_time,
       25 + 15,
       'Total time is set to the sum of all players time');
}

sub test_log_stats
{
    my ($fn) = @_;
    # set the modtime
    my @stat = stat $fn;
    SOF2Logger::ServerStats->log_modtime($stat[9]);

    is(SOF2Logger::ServerStats->log_modtime, $stat[9], 'modtime');
    $logparser->debug(0);
    my $fh = $logparser->get_checked_filehandle;
    is($fh, 0, 'modtimes match');
    is($logparser->parse_log, 
       -1, 
       'parse_log does not parse with files are the same');

    SOF2Logger::ServerStats->last_line('Bogus Last_line');

    # store new, different modtime which will force the logparser to 
    # search the logfile for the last line, which doesn't exist at this point
    SOF2Logger::ServerStats->log_modtime('123456');
    $fh = $logparser->get_checked_filehandle;
    my $line = <$fh>;
    chomp $line;
    is(
        $line, 
        ' 0:00 ------------------------------------------------------------', 
        'should be first line of log file'
    );
    eval {$logparser->parse_log};
    like($@, qr/unhandled line/, 'Tries to parse');

    $logparser->debug(1);

    SOF2Logger::ServerStats->last_line('  15:53 ClientBegin: 1');
    SOF2Logger::ServerStats->last_line_num(14);
    is(SOF2Logger::ServerStats->last_line,'  15:53 ClientBegin: 1', 'last line gets set on server_stats properly');
    is(SOF2Logger::ServerStats->last_line_num,14, 'last line num');
    $fh = $logparser->get_checked_filehandle;
    $line = <$fh>;
    chomp $line;
    is(
        $line,
        '  15:59 ClientUserinfoChanged: 1 n\h^7\t\3\identity\marinesoldier1\sir\0\sib\0\grade\0', 
        'lines match'
    );

    my $test_logparser = SOF2Logger::TEST::Testparser->new($log, 0);

    $test_logparser->parse_log;
    is(SOF2Logger::ServerStats->last_line, '  15:59 ClientBegin: 1', 'last line is set properly');
}

# simple class that over-writes unhandled_line in base class so we can run tests against it
{
    package SOF2Logger::TEST::Testparser;

    use base 'SOF2Logger::Parser';

    sub unhandled_line { return '' };

    sub shutdown_game { };

    1;
}

sub open_log
{
    my $filename = $$;
    open(TMP, '>', $filename) or
        die "Err: could not open filename $filename: $!\n";

    print TMP 
q( 0:00 ------------------------------------------------------------
   0:00 InitGame: \scorelimit\8\timelimit\20\sv_enableAdminPowers\1\sv_floodProtect\1\sv_maxPing\250\sv_minPing\0\sv_maxRate\7000\sv_hostname\          ^3NeW ^$BDK SeRVeR\dmflags\0\fraglimit\20\g_needpass\0\sv_privateClients\0\sv_maxclients\16\sv_punkbuster\1\sv_allowAnonymous\0\sv_allowDownload\0\g_maxGameClients\16\g_friendlyfire\1\sv_pure\1\sv_ospClient\1\version\SOF2MP GOLD V1.03 win-x86 Nov  5 2002\g_gametype\ctf\protocol\2004\mapname\mp_dust\gamename\SOF2MP\Config File\SOF2\Admin\Lt.Col.Hal Moore and SeeFor\g_available\22222222122222002222\gameversion\OSP v0.3i beta\g_plusplus\0
   0:00 Warmup:
   7:04 ClientConnect: 0 - 67.161.155.150:20100 [703FF6CB3DF8A87244D68A32E68601D2]
   7:04 ClientUserinfoChanged: 0 n\^}Res^$iNa^}toR ^$[BDK]^7\t\3\identity\marinesoldier1\sir\0\sib\0\grade\3
   7:11 ClientBegin: 0
   7:15 ClientUserinfoChanged: 0 n\^}Res^$iNa^}toR ^$[BDK]^7\t\3\identity\marinesoldier1\sir\0\sib\0\grade\3
   7:15 ClientBegin: 0
  15:30 say: ^}Res^$iNa^}toR ^$[BDK]^7: interested in 4:20? Check out www.the420clan.com for info and stats..
  15:33 ClientConnect: 1 - 24.161.158.30:20100 [7F9C10A5B2BE71E7563B774D375D8C71]
  15:33 ClientUserinfoChanged: 1 n\h^7\t\3\identity\marinesoldier1\sir\0\sib\0\grade\0
  15:53 ClientUserinfoChanged: 1 n\h^7\t\3\identity\marinesoldier1\sir\0\sib\0\grade\0
  15:53 ClientUserinfoChanged: 1 n\h^7\t\3\identity\marinesoldier1\sir\0\sib\0\grade\0
  15:53 ClientBegin: 1
  15:59 ClientUserinfoChanged: 1 n\h^7\t\3\identity\marinesoldier1\sir\0\sib\0\grade\0
  15:59 ClientBegin: 1
);
    close TMP;
    return $filename;

}

sub delete_log
{
    my $filename = shift;
    unlink $filename or 
        die "Err: could not delete '$filename': $!\n";
}

END {
    delete_log($$);
    unlink 'log_stats.dat';
}
