#!/usr/bin/perl

use strict;

use SOF2Logger::Player;
use SOF2Logger::Config;
use Test::More tests => 29;
use File::Basename qw/dirname/;

# grab the test config file from the config test files
my $config = SOF2Logger::Config->instance({config_file => dirname(__FILE__) . '/logparse.conf'});

test_creating_and_finding_new_players();
test_find_all();
test_delete();
test_find_alias();
test_freeze_thaw();

sub test_creating_and_finding_new_players {
  my $name = "ben^7";
  my $id = "ben";

  my $player = SOF2Logger::Player->create(id => $id, name => $name);
  is($player->name(), $name, 'name is correct');

  SOF2Logger::Player->create(id => 'foo', name => 'bar');

  my $found_player = SOF2Logger::Player->find($id);
  is($found_player->name(), $name, 'can find player');
}

sub test_find_all {
  SOF2Logger::Player->delete_all();
  SOF2Logger::Player->create( id => 'foo', name => 'bar');
  SOF2Logger::Player->create( id => 'foo2', name => 'bar2');

  my @players = SOF2Logger::Player->find_all();

  is(scalar @players, 2, 'num players is correct');
}

sub test_find_alias {
  my $player = SOF2Logger::Player->create( id => '0000000A', name => 'frank');
  is($player->id(), '0000000B', 'create aliases name properly');

  is(SOF2Logger::Player->find('0000000A')->id, '0000000B', 'find aliases correctly');
  is(SOF2Logger::Player->find('0000000B')->id, '0000000B', 'find has correct player-id');
}

sub test_delete {
  my $id = 123;
  my $player = SOF2Logger::Player->create( id => $id, name => 'bar');

  ok(SOF2Logger::Player->find($id), 'can find player before delete...');

  $player->delete();
  ok(!SOF2Logger::Player->find($id), '... but not after delete');
}

sub test_freeze_thaw {
  SOF2Logger::Player->create( id => 123, name => 'foo', total_time => 10, flag_returns => 1, rating => 1500 );
  SOF2Logger::Player->create( id => 456, name => 'bar', avatar_name => 'frank.jpg' );

  SOF2Logger::Player->freeze('test_freeze.txt');

  SOF2Logger::Player->delete_all();

  ok(!SOF2Logger::Player->find_all(), 'ensure no players');

  SOF2Logger::Player->thaw('test_freeze.txt');

  my $player = SOF2Logger::Player->find(123);

  is($player->name, 'foo', 'name is correct after thaw');
  is($player->total_time, 10, 'total_time is correct after thaw');
  is($player->flag_returns, 1, 'flag_returns is correct after thaw');
  is($player->flag_caps, 0, 'flag_caps is correct after thaw');
  is($player->rating, 1500, 'rating is correct' );

  my $p2 = SOF2Logger::Player->find(456);

  is($p2->name, 'bar', 'name');
  is($p2->avatar_name, 'frank.jpg', 'avatar_name');
  is($p2->flag_returns, 0, 'flag returns');

  unlink('test_freeze.txt');
}

my $name = "ben^7";
my $id = "ben";
my $player = SOF2Logger::Player->create( id => $id, name => $name);

isa_ok($player, 'SOF2Logger::Player::PlayerInst', 'ref correct');

is($player->name, $name, 'player name');
is($player->id, $id, 'player identifier');
is($player->total_time, 0, 'total time');

$player->total_time(15);
$player->add_time(12);

is($player->total_time, 27, 'adding total time');

$player->flag_caps(4);
$player->add_cap;

is($player->flag_caps, 5, 'flag captures');

# set up for a colorized name test
$name = "^-{^FLLC^-}^FO^-ver^FD^-rive^7";
$id = "{LLC}OverDrive";
$player = SOF2Logger::Player->create( id => $id, name => $name);

is($player->name, $name, 'player name with colors');

my $namefiltered = "{LLC}OverDrive";
my $avatar_name = "jedi_trainer.jpg";

$player = SOF2Logger::Player->create( id => $id, name => $name, avatar_name => $avatar_name);
is($player->id, $namefiltered, 'player name without colors');
is($player->avatar_name, $avatar_name, 'avatar name is correct');

$player->avatar_name('billy joe.jpg');
is($player->avatar_name, 'billy joe.jpg', 'can set avatar name');

my $bobby = SOF2Logger::Player->find_or_create(id => 1111, name => 'Bobby');
is($bobby->name, 'Bobby', 'Name for find_or_create is correct');

$bobby = SOF2Logger::Player->find_or_create(id => 1111, name => 'Frank');
is($bobby->name, 'Frank', 'find_or_create can also update info');
