#!/usr/bin/perl -w

use strict;

use Test::More tests => 17;

use SOF2Logger::Player;
use SOF2Logger::Config;
use File::Basename qw/dirname/;

BEGIN { use_ok('SOF2Logger::Utils',
               'create_get_set',
               'min_defined',
               'filename_for_player'); };

min_test();
test_getset();
test_strip_q3_colors_does_not_mangle_input_string();
test_strip_q3_colors();
test_clean_q3_color_string();
test_create_get_set();
test_filename_for_player();

sub test_create_get_set
{
    {
        package Foo::Bar;

        sub new
        {

            my $class = shift;
            my $self = {};
            $self->{Baz} = "nothing";
            bless $self, $class;
        }


        1;
    }

    my $method_name = 'Baz';
    my $foo = Foo::Bar->new();
    ok(!$foo->can($method_name), '$foo->' . $method_name . ' is not defined');
    create_get_set('Foo::Bar', $method_name);


    can_ok($foo, $method_name);

    is($foo->$method_name, "nothing", 'get portion of get_set works');

    $foo->$method_name('blah');

    is($foo->$method_name, 'blah', 'set portion of get_set works');
}

sub min_test
{
    my $smaller = 3;
    my $larger = 4;

    is(min_defined($smaller, $larger), $smaller, 'positive mins');

    my $neg_smaller = -2;
    my $neg_larger = -1;

    is(min_defined($neg_smaller, $neg_larger), $neg_smaller, 'negative mins');

    $neg_smaller = -1;
    my $zero = 0;

    is(min_defined($neg_smaller, $zero), $neg_smaller, 'negative and zero mins');

    is(min_defined($larger, undef), $larger, 'any undef values are ignored');
}

sub test_getset
{
    my $new = Test->new();

    is($new->{TEST_PROP}, -1, 'property before set');
    SOF2Logger::Utils->get_set($new, 'TEST_PROP', 4);
    is($new->{TEST_PROP}, 4, 'property after set');

    is(SOF2Logger::Utils->get_set($new, 'TEST_PROP', undef), 4, 'get value');
}

sub test_strip_q3_colors_does_not_mangle_input_string
{
    my $str = "^5This is a ^4Test ^7 String...";
    SOF2Logger::Utils->strip_q3_colors($str);

    is($str, "^5This is a ^4Test ^7 String...", "strings remain the same");
}

sub test_strip_q3_colors
{
    my $str = "^5This is a ^4Test ^7 String...";
    my $colorless_str = SOF2Logger::Utils->strip_q3_colors($str);

    is($colorless_str, "This is a Test  String...", "string has no color codes");   
}

sub test_clean_q3_color_string
{
    my $str = "^5This is a ^4Test ^7 String...";
    my $clean_str = SOF2Logger::Utils->clean_q3_color_string($str);

    is($clean_str, "thisisateststring", "string is clean");   

    is(SOF2Logger::Utils->clean_q3_color_string('^#^#^$0'), 'unknown', 'supplies unknown when appropriate');
}

sub test_filename_for_player {
  my $player = SOF2Logger::Player->find_or_create(id => 'Foobar', name => 'Baz');
  is(filename_for_player($player), 'Foobar.html', 'generated filename is correct');
}

{
    package Test;

    sub new
    {
        my ($class) = @_;
        my $self = {};
        $self->{TEST_PROP} = -1;
        return bless $self, $class;
    }
}

#  sub test_strip_colors
#  {
#      my $name = Utils->clean_q3_color_string(q/<f&o>o^<./);
#      is($name, '&lt;f&amp;o&gt;o<font color="#2CE02F">.</font>', 'stripping colors');

#      my $name2 = $parser->strip_colors(q/^-.^zhS^-.^z$^-nak^ze^7/);
#      my $name2_should_be = '<font color="#8A8789">.</font><font color="#EFD497">hS</font><font color="#8A8789">.</font><font color="#EFD497">$</font><font color="#8A8789">nak</font><font color="#EFD497">e</font>';
#      is($name2, $name2_should_be, 'stripping colors from complex names');

#      my $name3 = $parser->strip_colors(q/^3r^&K ^3| ^&x^jTaLeS^&x^7/);
#      my $name3_should_be = '<font color="#FFFF00">r</font><font color="#A39DC5">K </font><font color="#FFFF00">| </font><font color="#A39DC5">x</font><font color="#9BEE44">TaLeS</font><font color="#A39DC5">x</font>';
#      is($name3, $name3_should_be, 'stripping colors from complex names');

#      my $name4 = $parser->strip_colors(q/^w=U^1F^3F^w=^-WERWOLF^7/);
#      my $name4_should_be = '<font color="#000000">=U</font><font color="#FF0000">F</font><font color="#FFFF00">F</font><font color="#000000">=</font><font color="#8A8789">WERWOLF</font>';
#      is($name4, $name4_should_be, 'stripping colors from complex names');
#  }

