#!/usr/bin/perl

use strict;
use warnings;

BEGIN {
    use File::Basename qw/dirname/;
    unshift @INC, dirname($0), dirname($0) . '/lib';
}

use Carp;
use Cwd;
use File::Copy;
use Getopt::Long;
use SOF2Logger::Config;
use SOF2Logger::Kills;
use SOF2Logger::OutputHTML qw/
                              convert_seconds_to_time_format
                              html_authors_links
                              html_homepage_link
                              popular_weapon
                              player_stats_header

                              worst_enemy_header
                              worst_enemy_for_player
                              worst_enemy_footer

                              most_popular_targets_header
                              most_popular_targets_for_player
                              most_popular_targets_footer

                              most_popular_weapons_header
                              most_popular_weapons_for_player
                              most_popular_weapons_footer

                              kills_by_weapon_players

                              main_stats_header
                              main_stats_for_player
                              main_stats_footer

                              htmlize_and_link_player
                              /;
use SOF2Logger::ParserBuilder;
use SOF2Logger::Player;
use SOF2Logger::ServerStats;
use SOF2Logger::Utils qw/min_defined show_do filename_for_player/;
use Memoize;
use Text::Template;

# $SIG{'__DIE__'} = sub { confess  @_ };

# this script has a homepage, it is:
my $HOME_PAGE= 'http://gamestats.sf.net';
my $id = q$Id: gamestats.pl 288 2006-04-26 05:36:51Z bprew $;
$id =~ /\.pl (\d+)/;
my $VERSION = "1.5.$1";

memoize('players_list_with_main_stat_info');

my %AUTHORS = (
    'Sam Evans'       => 'sam@neuroflux.com',
    'Jesse "miCro"'   => 'go_jesse@yahoo.com',
    'Ravencroft'      => 'ravencroft@clannumbnutz.com',
    'MatrixCaptain'   => 'bprew@sf.net',
    'Yewbie (ROCMOD)' => 'yewbie@theborderpatrol.com',
);


## VROOM!  Here we go.. ##

my %command_line_switches;

GetOptions(
    "c=s"       => \$command_line_switches{config_file},
    "reinit!"   => \$command_line_switches{reinit},
    "d!"        => \$command_line_switches{debug},
    "h!"        => \$command_line_switches{help}
) or usage();

($command_line_switches{log_file} = shift @ARGV) if @ARGV;

my $config = SOF2Logger::Config->instance(\%command_line_switches);

usage() if $config->help;

# The config shouldn't do this, but it should provide the information
# this thing sometimes leaves the '.html' file laying around, but oh well.
$config->delete_old_player_files();

# Set OutputHTML's environment
$SOF2Logger::OutputHTML::image_dir = $config->image_dir;

read_stored_stats_into_hashes();

show_do( "Parsing logfile", \&parse_log );

# Attempt to copy resources after parsing the log, as the log can
# determine what type of game it is.
$config->check_and_copy_resources();

write_player_data($config->player_file);

print "limiting players displayed to kills > "
. $config->minimum_kills
. " and time > "
. $config->minimum_time . "\n";

print 'Removed '
. remove_players_under_minimum($config, SOF2Logger::Player->find_all())
. ' players because they did not meet the minimum time or kills, or they '
. "were explicitly filtered in player.filter\n";

########################################################################
# All done parsing things, now create the html                         #
########################################################################

# These only need to be done ONCE
$Q::TOP_WEAPON_USERS = top_weapon_handler($config);
$Q::popular_weapon =
popular_weapon(SOF2Logger::Kills->total_kills,
               SOF2Logger::Kills->kills_by_weapon('only_unfiltered_players'));
$Q::AUTHORS = html_authors_links(\%AUTHORS);
$Q::administrator = SOF2Logger::ServerStats->server_admin;
$Q::admin_email   = SOF2Logger::ServerStats->server_admin_email;
$Q::servername    = SOF2Logger::ServerStats->server_name;
$Q::total_kills = SOF2Logger::Kills->total_kills;
$Q::min_kills = $config->minimum_kills;
$Q::min_time = convert_seconds_to_time_format($config->minimum_time);
$Q::total_time =
convert_seconds_to_time_format(SOF2Logger::ServerStats->total_time);
$Q::now = scalar localtime;
$Q::num_players = scalar SOF2Logger::Player->find_all();
$Q::total_lines = SOF2Logger::ServerStats->total_lines();

make_main_stats();
make_individual_stats();

backup_logfile();

# if its a full path print the right thing
if (($config->output_dir =~ m/^\w:/) || ($config->output_dir =~ m/^\//)) {
    print "SUCCESS!  Look in " . $config->output_dir . " for the generated HTML\n";
} else {
    print "SUCCESS!  Look in " . cwd() . '/' . $config->output_dir . " for the generated HTML\n";
}

################################################################################
################################################################################
#                                     SUBROUTINES
################################################################################
################################################################################

sub read_stored_stats_into_hashes {
    SOF2Logger::Player->thaw($config->player_file);
    SOF2Logger::ServerStats->thaw($config->server_stats_file);
    SOF2Logger::Kills->thaw($config->kills_file);
}

sub parse_log {
    # If we haven't specified a game_type, default to q3, as it will be
    # able to adjust itself to whatever logfile we really have
    my $game_type = $config->game_type ? $config->game_type : 'Q3A';

    my $parser = SOF2Logger::ParserBuilder->get_parser( $game_type,
                                                        $config->game_mod );

    $parser->parse_log;
}

################################################################################
=item write_player_data
Write the hash information to the player datafile before we munge the data.
=cut
sub write_player_data {
    SOF2Logger::Player->freeze($config->player_file);
    SOF2Logger::ServerStats->freeze($config->server_stats_file);
    SOF2Logger::Kills->freeze($config->kills_file);
}

################################################################################
=item remove_players_under_minimum CONFIG
Prunes players from our lists that fall under min kills or min time.  Returns 
the number of players filtered.
=cut
sub remove_players_under_minimum {
    my ($config, @players) = @_;
    my $count = 0;

    foreach my $player (@players) {
        if ($config->is_player_filtered($player)
            || $player->total_time < $config->minimum_time
            || SOF2Logger::Kills->kills_for_player($player->id) < $config->minimum_kills) {
            $player->is_filtered(1);
            SOF2Logger::Kills->is_filtered_player($player->id, 1);
            $count++;
        }
    }

    return $count;
}

sub players_list_with_main_stat_info {
    my %player_info;

    my %best_of;
    foreach my $player (SOF2Logger::Player->find_all()) {

        next if $player->is_filtered;

        my $id = $player->id;
        my $kills = SOF2Logger::Kills->kills_for_player($id);
        my $suicides = SOF2Logger::Kills->suicides_for_player($id);
        my $deaths = SOF2Logger::Kills->deaths_for_player($id);
        my $time = $player->total_time;
        my $flag_caps = $player->flag_caps;
        my $flag_returns = $player->flag_returns;

        my $kills_per_hour = $time
        ? int($kills / ($time/3600))
        : 0;

        # efficiency is 100 * (kills / (kills + deaths + suicides))
        # but we ignore suicides for JK2
        my $efficiency = 100 * (
            $kills /
            (($kills + $deaths + (($config->game_type eq 'JK2') ? 0 : $suicides)) || 1));

        $player_info{$id} =
        { rating => $player->rating,
          kills => $kills,
          kills_per_hour => $kills_per_hour,
          deaths => $deaths,
          suicides => $suicides,
          efficiency => $efficiency,
          time => $time,
          flag_caps => $flag_caps,
          flag_returns => $flag_returns,
          player => $player,
        };

        foreach my $best_of_stat (qw/rating kills deaths suicides time kills_per_hour/) {
            $best_of{$best_of_stat} = $id if !$best_of{$best_of_stat}
            || $player_info{$id}->{$best_of_stat} > $player_info{ $best_of{$best_of_stat} }->{$best_of_stat};
        }
    }

    foreach my $best_of_stat (keys %best_of) {
        my $var = 'Q::' . "player_with_most_$best_of_stat";

        no strict 'refs';

        ${ $var } = htmlize_and_link_player( SOF2Logger::Player->find($best_of{$best_of_stat}),
                                             $config->player_data_dir );

    }
    return values %player_info;
}

################################################################################
=item top_weapon_handler CONFIG KILLS

Loop through all the weapons and determine which players have used
that weapon and how many times.

=cut
sub top_weapon_handler
{
    my ($config) = @_;
    my $limit = $config->top_weap;
    my $weapon_handler_html;

    foreach my $weapon_name (@{SOF2Logger::Kills->weapons}) {
        # convert from ids to actual player objects
        my @players =
        map { [ SOF2Logger::Player->find($_->[0]), $_->[1] ]}
        @{SOF2Logger::Kills->kills_by_player_for_weapon($weapon_name)};

        # ignore weapons that have no kills
        next unless @players;

        # make all players with kills for weapon
        $weapon_handler_html .= kills_by_weapon_players(
            SOF2Logger::Kills->kills_for_weapon($weapon_name),
            $limit,
	    $weapon_name,
            $config,
            @players,
        );

    }
    return $weapon_handler_html;
}

################################################################################
sub make_main_stats {
    my $template = Text::Template->new( TYPE => 'FILE',
                                        SOURCE => $config->main_template );

    $Q::STYLE_SHEET = $config->css_dir . '/' . $config->stylesheet;
    $Q::IMAGE_NAME = join('/', $config->image_dir,
                          $config->game_type eq 'JK2'
                          ? 'JK2.jpg'
                          : 'sof2logger.gif');

    $Q::HOME_PAGE = $HOME_PAGE;
    $Q::VERSION = $VERSION;

    # Create the configuration mappings for easy stats creation
    my %CONF_MAP = ( kills          => $config->output_file,
                     kills_per_hour => $config->kph_file,
                     deaths         => $config->deaths_file,
                     suicides       => $config->suicides_file,
                     efficiency     => $config->eff_file,
                     time           => $config->timeplayed_file,
                     flag_caps      => $config->captures_file,
                     flag_returns   => $config->returns_file);

    my @template_files;

    while ( my ($stat_name, $output_file) = each %CONF_MAP ) {
	warn "making $stat_name page\n";

	$output_file = $config->output_dir . '/' . $output_file;

	my @sorted_players = sort { $b->{$stat_name} <=> $a->{$stat_name} }
        grep { !($_->{player})->is_filtered } players_list_with_main_stat_info();

	next unless @sorted_players;

	$Q::PLAYER_STATS = player_stats(@sorted_players);

	open(OUTPUT, '>', $output_file)
        or die "Err: could not open template file '$output_file' : $!";

	$template->fill_in( PACKAGE => 'Q', OUTPUT => \*OUTPUT );

	if ($Text::Template::ERROR) {
            warn "error filling in template " . $Text::Template::ERROR;
	}

	close(OUTPUT);
    }
}

################################################################################
=item player_stats STAT_TYPE

Short:  The Player stats in HTML format

=cut
sub player_stats {
    my (@sorted_players) = @_;
    my $playerstring = main_stats_header($config);
    my $count = 1;

    for my $p_info (@sorted_players) {
        last unless $count <= $config->top_num;
        next if $p_info->{player}->is_filtered;

        $playerstring .=
        main_stats_for_player( $count,
                               $p_info->{player},
                               $config->player_data_dir,
                               $p_info->{kills},
                               $p_info->{deaths},
                               $p_info->{suicides},
                               $p_info->{flag_caps},
                               $p_info->{flag_returns},
                               $p_info->{time},
                               $p_info->{kills_per_hour},
                               $p_info->{player}->rating,
                               $p_info->{efficiency} );

        $count++;
    }

    $playerstring .= main_stats_footer();
    return $playerstring;
}

################################################################################
# Short: Makes all the html for each player
################################################################################
sub make_individual_stats {

    my $template = Text::Template->new( TYPE => 'FILE',
                                        SOURCE => $config->player_template );

    $P::style_sheet = '../' . $config->css_dir .'/' . $config->stylesheet;

    foreach my $p_info (players_list_with_main_stat_info()) {
	my $player = $p_info->{player};
        next if $player->is_filtered;

        $P::player_vs_player = I_player_vs_player($player);
        $P::worst_enemy = I_worst_enemy($player);
        $P::popular_weapon = I_popular_weapon($player);
	$P::player_stats = I_player_stats($p_info);

        #convert to a good file name
        my $output_file = join("/",
                               $config->output_dir(),
                               $config->player_data_dir(),
                               filename_for_player($player));

        if (-e ($output_file)) {
            warn "Err: output file '$output_file' exists\n";
            substr($output_file, -5, 3) = '_1_';
        }


        # Write out the player's profile
        open (PLAYERHTMLOUT, '>', $output_file) or
        die("Could not open player file ($output_file) for write: $!");

        $template->fill_in( PACKAGE => 'P', OUTPUT => \*PLAYERHTMLOUT);
        close(PLAYERHTMLOUT);
    }
}

################################################################################
=item I_player_vs_player PLAYER

Short: HTML-ize the most popular targets for a player

=cut
sub I_player_vs_player {
    my ($player) = @_;

    my $limit = $config->top_weap;
    my $count = 1;
    my $pop_victim = most_popular_targets_header($player->name);
    my $victim_info = SOF2Logger::Kills->kills_by_player_for_player($player->id);
    my $total_kills = SOF2Logger::Kills->kills_for_player($player->id);

    foreach my $info (@$victim_info) {
        my ($vic_id, $deaths) = @$info;
        my $victim = SOF2Logger::Player->find($vic_id);
        last if $count > $limit;

        unless ($victim) {
            print "$vic_id does not exist\n";
            next;
        }

        next if $victim->is_filtered;

        $pop_victim .= most_popular_targets_for_player(
            $count,
            $victim,
            $deaths,
            $total_kills,
            1);

        $count++;
    }
    $pop_victim .= most_popular_targets_footer();

    return $pop_victim;
}

################################################################################
=item I_worst_enemy PLAYER

The worst enemy for an individual player

=cut
sub I_worst_enemy {
    my ($player) = @_;
    my $worst_enemy_html = worst_enemy_header($player->name);
    my $count = 1;
    my $enemies_info = SOF2Logger::Kills->deaths_by_player_for_player($player->id);
    my $total_deaths = SOF2Logger::Kills->deaths_for_player($player->id);
    my $limit = $config->top_weap;
    my $depth = 1;

    foreach my $info (@$enemies_info) {
        my ($enemy_id, $deaths) = @$info;

        last if $count > $limit;

        next if SOF2Logger::Player->find($enemy_id)->is_filtered;

        $worst_enemy_html .= worst_enemy_for_player(
            $count,
            SOF2Logger::Player->find($enemy_id),
            $deaths,
            $total_deaths,
            $depth);

        $count++;
    }
    $worst_enemy_html .= worst_enemy_footer();
    return $worst_enemy_html;
}

################################################################################
=item I_popular_weapon PLAYER

Most popular weapon for an individual player

=cut
sub I_popular_weapon
{
    my ($player) = @_;
    my $count = 1;
    my $depth = 1;
    my $limit = $config->top_weap;
    my $weapon_info = SOF2Logger::Kills->kills_by_weapon_for_player($player->id);
    my $total_kills = SOF2Logger::Kills->kills_for_player($player->id);

    my $pop_weapon = most_popular_weapons_header($player->name);

    for my $info (@$weapon_info) {
        my ($weapon, $times_used) = @$info;

        $pop_weapon .= most_popular_weapons_for_player(
            $count,
            $weapon,
            $times_used,
            $total_kills,
            $depth);

        $count++;
    }

    $pop_weapon .= most_popular_weapons_footer();
    return $pop_weapon;
}

################################################################################
=item I_player_stats PLAYER

The Header HTML on the individual player stat pages

=cut
sub I_player_stats
{
    my ($p_info) = @_;

    my $headshot = SOF2Logger::Kills->headshots_for_player($p_info->{player}->id);

    my $player_stats = player_stats_header(
        $config,
        $p_info->{player}->name,
        $p_info->{kills},
        $p_info->{deaths},
        $p_info->{suicides},
        $p_info->{flag_caps},
        $p_info->{flag_returns},
        $p_info->{player}->rating,
        $headshot,
        $p_info->{efficiency},
        $p_info->{time},
        $p_info->{player}->avatar_name);

    return $player_stats;
}

################################################################################
=item backup_logfile
Backs up the original logfile to 'date_logfile'
Will first attempt to rename the file and if that fails will attempt to
copy the file.  If that fails, it notifies the user and does nothing.
=cut
sub backup_logfile
{
    if ($config->backup) {
        my (
            undef,
            undef,
            undef,
            $day,
            $month,
            $year,
            undef,
            undef,
            undef
        ) = localtime;

        my $log = $config->log_file;
        my $new_filename;

        if ($log =~ /(\.\w{3})$/) {
            $new_filename = sprintf "%s_%d-%02d-%2d%s", 
            substr($config->log_file, 0, length($config->log_file) - 4), 
            $year + 1900, 
            $month, 
            $day,
            $1;
        } else {
            $new_filename = sprintf "%s_%d-%02d-%2d", $config->log_file, $year + 1900, $month, $day;
        }

        print "Copyfile logfile ". $config->log_file ." to $new_filename\n";
        if (-f $new_filename) {
            print "Err: $new_filename exists, using $new_filename(1) instead\n";
            $new_filename .= '(1)';
        }
        if (!rename($config->log_file, $new_filename)) {
            print "Err: I could not rename ".$config->log_file." to $new_filename: $!\n";
            print "I will try to copy the file instead of renaming it\n";
            copy($config->log_file, $new_filename) or
            print "Err: Arrgh! I couldn't even COPY ".$config->log_file." to $new_filename: $!\n" .
            "I guess you'll have to do it yourself.  Sorry about that\n";
        }
    }
}

sub usage
{
    print qq/
VERSION : $VERSION

NAME
    $0 - Soldier of Fortune 2 Server Log Processing Program

PREREQUISITES
    Perl 5.003 or greater

PUBLIC INTERFACE

logparsesof2.pl  [-c config_file] [-reinit|-r] [-h] [-d] LOGFILE
    -c config_file
        Specify which config file the script should use. 
        ex. logparsesof2.pl -c foo.conf 

    -reinit | -r
        Use this option to delete and reinitialize (reset) the player datafile

    -h
        Use this option to display this screen.  (Help Screen)

    -d
        Enable debug mode from the command line
        (lots of output, you are warned!)

    LOGFILE
        Use this option to specify an alternate logfile.  This will
        override the LOGFILE directive in the .conf file

AUTHORS

@{[ join "\n", map { "$_ (" . $AUTHORS{$_} . ")" } keys %AUTHORS]}

HOMEPAGE : $HOME_PAGE
/;
    exit(1);
}

