#!/usr/bin/perl

use strict;
use warnings;

use File::Basename qw/basename/;
use File::Temp qw/tempdir/;
use Getopt::Long;
use Net::FTP;

GetOptions("h|help" => \my $help,
           "r|revision=s" => \my $revision,
           "b|batch" => \my $batch,
           "test" => \my $test_mode)
  or usage();

usage() if $help || !$revision;

my $os_options =
  { 'linux'   => { 'exe_name' => 'gamestats',
                   'rel_name' => "gamestats-linux-$revision.tgz",
                   'zip_command' => 'tar zcvf $release_file gamestats', },

    'cygwin'  => { 'exe_name' => 'gamestats.exe',
                   'rel_name' => "gamestats-windows-$revision.zip",
                   'zip_command' => "7z a -r -tzip $release_file gamestats", },

    'MSWin32' => { 'exe_name' => 'gamestats.exe',
                   'rel_name' => "gamestats-windows-$revision.zip",
                   'zip_command' => "7z a -r -tzip $release_file gamestats", },
  };


if ($batch) {
  require Net::SSH;
  my @hosts = qw/Ben@192.168.0.3 benprew@10.0.0.25/;

  # cvs revisions have to start with a letter
  $revision = "r$revision"
    if ($revision =~ /^\d/);

  unless ($test_mode) {
    warn "Tagging cvs repo with revision $revision\n";
    `cvs rtag $revision gamestats`;
  }

  foreach my $host (@hosts) {
    warn "copying script to $host\n";
    `scp $0 $host:bin/`;

    warn "running make_release -r $revision on $host\n";
    run_make_release_on_host($host, $revision, $test_mode);
  }
} else {
  my $sf_ftp = 'upload.sourceforge.net';
  my $release_file = $os_options->{$^O}{rel_name};
  my $dir = $test_mode
    ? tempdir() : tempdir(CLEANUP => 1);

  chdir $dir or die "could not cd to $dir\n";

  warn "doing cvs checkout\n";
  `cvs -d :ext:bprew\@cvs.sf.net:/cvsroot/gamestats export -r HEAD gamestats`;

  warn "running par\n";
  my $exe_name = $os_options->{$^O}{exe_name};
  `pp -o gamestats/$exe_name gamestats/logparsesof2.pl`;

  warn "running $zip_command_line\n";
  system($zip_command_line) == 0
    or warn "Err: could not exec '$zip_command_line': $?\n";

  unless ($test_mode) {
    warn "Uploading $release_file to $sf_ftp\n";
    upload_file($release_file, $sf_ftp);
  };

  warn "Done! removing temp files\n";
}

sub run_make_release_on_host {
  my ($host, $release, $test_mode) = @_;
  $test_mode = $test_mode ? '--test' : '';
  Net::SSH::ssh($host, "PATH=~/bin:/cygdrive/c/Perl/bin:\$PATH; " . basename($0) . " -r $revision $test_mode");
}

sub upload_file {
  my ($release_file, $ftp_host) = @_;

  my $ftp = Net::FTP->new($ftp_host)
    or die "Err: cannot connect to $ftp_host: $@";

  $ftp->login('anonymous', 'bprew@sf.net')
    or die "Err: cannot login ", $ftp->message;

  $ftp->cwd('incoming/')
    or die "Err: cannot cwd to incoming/ ", $ftp->message;

  $ftp->binary;

  $ftp->put($release_file)
    or die "Err: cannot put $release_file ", $ftp->message;

  $ftp->quit;
}


sub usage {
  print "\n", basename($0), qq/ [-r|--revision revision]

Release the current revisions of gamestats to sourceforge.  This
script checks out the gamestats module from the sf cvs server, runs
par (pp) on it, and uploads a zip file to sourceforge.

-r, --revision
    required! specifiy a revision number for the release.  The
    cvs repo will be tagged with the revision after the files are
    uploaded.

-b, --batch Run make_release in "batch" mode where it will connect to
    the various hosts required to release the various version of
    gamestats and have them upload their version of the
    gamestats-release
/;
  exit(-1);
}
